@@ -1,4 +1,4 @@
-"""This file provides the Expr class.  This attempts to create a
+"""This module provides the Expr class.  This attempts to create a
 consistent interface to SymPy's expressions.
 
 Copyright 2014--2019 Michael Hayes, UCECE
@@ -101,20 +101,13 @@ class ExprContainer(object):
     
 class ExprMisc(object):
 
-    @property    
     def pdb(self):
         """Enter the python debugger."""
         
         import pdb; pdb.set_trace()
         return self
 
-    def debug(self):
-
-        name = self.__class__.__name__
-        s = '%s(' % name
-        print(symdebug(self.expr, s , len(name) + 1))
-
-    
+        
 class ExprDict(ExprPrint, ExprContainer, ExprMisc, OrderedDict):
 
     """Decorator class for dictionary created by sympy."""
@@ -146,6 +139,23 @@ class ExprDict(ExprPrint, ExprContainer, ExprMisc, OrderedDict):
             new[k] = simplify(v)
         return new
 
+    def subs(self, *args, **kwargs):
+        """Substitute variables in expression, see sympy.subs for usage."""
+
+        new = self.__class__()
+        for k, v in self.items():
+            try:
+                k = k.subs(*args, **kwargs)
+            except:
+                pass
+            try:
+                v = v.subs(*args, **kwargs)
+            except:
+                pass            
+                
+            new[k] = v
+        return new
+
     
 class ExprList(ExprPrint, list, ExprContainer, ExprMisc):
     """Decorator class for list created by sympy."""
@@ -154,31 +164,35 @@ class ExprList(ExprPrint, list, ExprContainer, ExprMisc):
     # in preference to list's one.  Alternatively, add explicit
     # _repr_pretty_ method here.
     
-    def __init__(self, arglist, evalf=False):
+    def __init__(self, iterable, evalf=False):
 
-        eargs = []
-        for item in arglist:
+        eiterable = []
+        for item in iterable:
             if evalf:
                 try:
-                    item = item.evalf
+                    item = item.evalf()
                 except:
                     pass
-            eargs.append(item)
+            else:
+                item = expr(item)
+            eiterable.append(item)
         
-        super (ExprList, self).__init__(eargs)
+        super (ExprList, self).__init__(eiterable)
 
     def subs(self, *args, **kwargs):
         """Substitute variables in expression, see sympy.subs for usage."""
         
         return expr([e.subs(*args, **kwargs) for e in self])
         
-    
+
 class ExprTuple(ExprPrint, tuple, ExprContainer, ExprMisc):
     """Decorator class for tuple created by sympy."""
 
-    def __init__(self, arglist):
-        eargs = [expr(e) for e in arglist]
-        super (ExprTuple, self).__init__(tuple(eargs))
+    # Tuples are immutable, need to use __new__
+    def __new__(cls, iterable):
+
+        eiterable = [expr(e) for e in iterable]
+        return super (ExprTuple, cls).__new__(cls, eiterable)
 
     def subs(self, *args, **kwargs):
         """Substitute variables in expression, see sympy.subs for usage."""
@@ -223,7 +237,7 @@ class Expr(ExprPrint, ExprMisc):
             self.assumptions = assumptions.copy()
             self.expr = arg.expr
             return
-            
+
         # Perhaps could set dc?
         if arg == 0:
             assumptions['causal'] = True
@@ -326,6 +340,33 @@ class Expr(ExprPrint, ExprMisc):
             return False
         return self.assumptions['complex'] == True
 
+    @property
+    def is_conditional(self):
+        """Return True if expression has a condition, such as t >= 0."""
+        
+        expr = self.expr
+        # Could be more specific, such as self.var >= 0, but might
+        # have self.var >= t1.
+        return expr.is_Piecewise
+
+    @property
+    def is_rational_function(self):
+        """Return True if expression is a rational function."""
+
+        return self.expr.is_rational_function(self.var)
+    
+    @property
+    def is_strictly_proper(self):
+        """Return True if the degree of the dominator is greater
+        than the degree of the numerator.
+        This will throw an exception if the expression is not a
+        rational function."""
+
+        if self._ratfun is None:
+            return False
+        
+        return self._ratfun.is_strictly_proper
+    
     @property
     def val(self):
         """Return floating point value of expression if it can be evaluated,
@@ -333,21 +374,12 @@ class Expr(ExprPrint, ExprMisc):
 
         return self.evalf()
 
-    @property
     def evalf(self):
         """Return floating point value of expression if it can be evaluated,
         otherwise the expression."""
 
         return self.expr.evalf()    
 
-    @property
-    def omega(self):
-        """Return angular frequency."""
-
-        if 'omega' not in self.assumptions:
-            return omegasym
-        return self.assumptions['omega']
-
     def __hash__(self):
         # This is needed for Python3 so can create a dict key,
         # say for subs.
@@ -409,6 +441,23 @@ class Expr(ExprPrint, ExprMisc):
         
         return wrap
 
+    def debug(self):
+        """Print the SymPy epxression and the assumptions for all symbols in
+        the expression."""
+
+        name = self.__class__.__name__
+        s = '%s(' % name
+        print(symdebug(self.expr, s, len(name) + 1))
+
+    @property
+    def func(self):
+        """Return the top-level function in the Sympy Expression.
+
+        For example, this returns Mul for the expression `3 * s`.
+        See also .args(), to return the args, in this case `(3, s)`"""
+
+        return self.expr.func
+    
     def __abs__(self):
         """Absolute value."""
 
@@ -560,6 +609,13 @@ class Expr(ExprPrint, ExprMisc):
     def __add__(self, x):
         """Add"""
 
+        # Convert Vs + Vt -> Voltage, etc.
+        if (hasattr(self, 'superkind') and hasattr(x, 'superkind') and
+            self.__class__ != x.__class__ and self.superkind ==
+            x.superkind):
+            cls = {'Voltage' : Voltage, 'Current' : Current}[self.superkind]
+            return cls(self) + cls(x)
+        
         cls, self, x, assumptions = self.__compat_add__(x, '+')
         return cls(self.expr + x.expr, **assumptions)
 
@@ -578,13 +634,19 @@ class Expr(ExprPrint, ExprMisc):
     def __sub__(self, x):
         """Subtract"""
 
+        # Convert Vs - Vt -> Voltage, etc.
+        if (hasattr(self, 'superkind') and hasattr(x, 'superkind') and
+            self.__class__ != x.__class__ and self.superkind ==
+            x.superkind):            
+            cls = {'Voltage' : Voltage, 'Current' : Current}[self.superkind]
+            return cls(self) - cls(x)        
+
         cls, self, x, assumptions = self.__compat_add__(x, '-')
         return cls(self.expr - x.expr, **assumptions)
 
     def __pow__(self, x):
         """Pow"""
 
-        # TODO: FIXME
         cls, self, x, assumptions = self.__compat_mul__(x, '**')
         return cls(self.expr ** x.expr, **assumptions)
 
@@ -593,7 +655,7 @@ class Expr(ExprPrint, ExprMisc):
 
         # TODO: FIXME
         cls, self, x, assumptions = self.__compat_mul__(x, '**')
-        return cls(self.expr ** x.expr, **assumptions)    
+        return cls(x.expr ** self.expr, **assumptions)    
 
     def __or__(self, x):
         """Parallel combination"""
@@ -693,6 +755,10 @@ class Expr(ExprPrint, ExprMisc):
 
         return cls(self.expr * x.expr / (self.expr + x.expr), **assumptions)
 
+    def copy(self):
+        """Copy the expression."""
+        return self.__class__(self.expr, **self.assumptions)
+
     @property
     def conjugate(self):
         """Return complex conjugate."""
@@ -729,7 +795,19 @@ class Expr(ExprPrint, ExprMisc):
 
     @property
     def _ratfun(self):
-        return Ratfun(self.expr, self.var)
+
+        try:
+            return self.__ratfun
+        except:
+            pass
+
+        if self.var is None:
+            self.__ratfun = None
+        else:
+            # Note, this handles expressions that are products of
+            # rational functions and arbitrary delays.  
+            self.__ratfun = Ratfun(self.expr, self.var)
+        return self.__ratfun
 
     @property
     def K(self):
@@ -747,17 +825,14 @@ class Expr(ExprPrint, ExprMisc):
     def D(self):
         """Return denominator of rational function."""
 
-        if self.var is None:
-            return self.__class__(1)
-        
         return self.denominator
 
     @property
     def numerator(self):
         """Return numerator of rational function."""
 
-        if self.var is None:
-            return self
+        if self._ratfun is None:
+            return self.copy()
 
         return self.__class__(self._ratfun.numerator)
 
@@ -765,6 +840,9 @@ class Expr(ExprPrint, ExprMisc):
     def denominator(self):
         """Return denominator of rational function."""
 
+        if self._ratfun is None:
+            return self.__class__(1)
+        
         return self.__class__(self._ratfun.denominator)
 
     def rationalize_denominator(self):
@@ -790,14 +868,36 @@ class Expr(ExprPrint, ExprMisc):
 
         return N / D
 
+    def factor_const(self):
+
+        from .utils import factor_const
+
+        c, r = factor_const(self, self.var)
+        return cExpr(c), self.__class__(r, **self.assumptions)
+
+    def term_const(self):
+
+        from .utils import term_const
+
+        c, r = term_const(self, self.var)
+        return cExpr(c), self.__class__(r, **self.assumptions)    
+
     def multiply_top_and_bottom(self, factor):
         """Multiply numerator and denominator by common factor."""
 
-        N = (self.N * factor).expand()
-        D = (self.D * factor).expand()
+        N = self.N.expr
+        D = self.D.expr
 
-        return N / D    
+        return self.__class__(sym.Mul(sym.Mul(N, factor, evaluate=False),
+                                      1 / sym.Mul(D, factor, evaluate=False),
+                                      evaluate=False))
     
+    def merge_assumptions(self, **assumptions):
+        
+        new_assumptions = self.assumptions.copy()
+        new_assumptions.update(assumptions)
+        return new_assumptions
+
     @property
     def magnitude(self):
         """Return magnitude"""
@@ -910,6 +1010,8 @@ class Expr(ExprPrint, ExprMisc):
         This is replaced by arg and then evaluated to obtain a result.
         """
 
+        is_causal = self.is_causal
+        
         def evaluate_expr(expr, var, arg):
 
             # For some reason the new lambdify will convert a float
@@ -956,13 +1058,23 @@ class Expr(ExprPrint, ExprMisc):
             # For negative arguments, np.sqrt will return Nan.
             # np.lib.scimath.sqrt converts to complex but cannot be used
             # for lamdification!
-            func = lambdify(var, expr,
+            func1 = lambdify(var, expr,
                             ({'DiracDelta' : dirac,
                               'Heaviside' : heaviside,
                               'UnitImpulse' : unitimpulse,
                               'sqrt' : sqrt, 'exp' : exp},
                              "scipy", "numpy", "math", "sympy"))
 
+            def func(arg):
+                # Lambdify barfs on (-1)**n if for negative values of n.
+                # even if have (-1)**n * Heaviside(n)
+                # So this function heads Lambdify off at the pass,
+                # if the function is causal.
+                
+                if is_causal and arg < 0:
+                    return 0
+                return func1(arg)
+
             try:
                 result = func(arg0)
                 response = complex(result)
@@ -1173,6 +1285,21 @@ class Expr(ExprPrint, ExprMisc):
         ret = symsimplify(self.expr)
         return self.__class__(ret, **self.assumptions)
 
+    def replace(self, query, value, map=False, simultaneous=True, exact=None):
+
+        try:
+            query = query.expr
+        except:
+            pass
+
+        try:
+            value = value.expr
+        except:
+            pass        
+
+        ret = self.expr.replace(query, value, map, simultaneous, exact)
+        return self.__class__(ret, **self.assumptions)        
+        
     def subs(self, *args, **kwargs):
         """Substitute variables in expression, see sympy.subs for usage."""
 
@@ -1268,44 +1395,57 @@ class Expr(ExprPrint, ExprMisc):
         symdict.update(funcdict)
         return symdict
 
+    def _fmt_roots(self, roots, aslist=False):
+
+        if not aslist:
+            rootsdict = {}
+            for root, n in roots.items():
+                rootsdict[expr(root)] = n
+            return expr(rootsdict)
+            
+        rootslist = []
+        for root, n in roots.items():
+            rootslist += [expr(root)] * n        
+        return expr(rootslist)        
+    
     def roots(self, aslist=False):
         """Return roots of expression as a dictionary
         Note this may not find them all."""
 
-        roots = self._ratfun.roots()
-        if not aslist:
-            return expr(roots)
-        rootslist = []
-        for root, count in roots.items():
-            rootslist += [root] * count
-        return expr(rootslist)
+        if self._ratfun is None:
+            roots = {}
+        else:
+            roots = self._ratfun.roots()
+        return self._fmt_roots(roots, aslist)        
             
     def zeros(self, aslist=False):
         """Return zeroes of expression as a dictionary
         Note this may not find them all."""
 
-        return self.N.roots(aslist)
+        if self._ratfun is None:
+            zeros = {}
+        else:
+            zeros = self._ratfun.zeros()
+        return self._fmt_roots(zeros, aslist)        
 
     def poles(self, aslist=False, damping=None):
         """Return poles of expression as a dictionary
         Note this may not find them all."""
 
-        poles = self._ratfun.poles(damping=damping)
+        if self._ratfun is None:
+            return self._fmt_roots({}, aslist)            
         
-        if not aslist:
-            polesdict = {}
-            for pole in poles:
-                key = pole.expr
-                if key in polesdict:
-                    polesdict[pole.expr] += pole.n
-                else:
-                    polesdict[pole.expr] = pole.n
-            return expr(polesdict)
-            
-        poleslist = []
+        poles = self._ratfun.poles(damping=damping)
+
+        polesdict = {}
         for pole in poles:
-            poleslist += [pole.expr] * pole.n
-        return expr(poleslist)
+            key = pole.expr
+            if key in polesdict:
+                polesdict[key] += pole.n
+            else:
+                polesdict[key] = pole.n        
+
+        return self._fmt_roots(polesdict, aslist)
 
     def canonical(self, factor_const=False):
         """Convert rational function to canonical form (aka polynomial form);
@@ -1323,7 +1463,8 @@ class Expr(ExprPrint, ExprMisc):
         See also general, partfrac, standard, timeconst, and ZPK
 
         """
-
+        if self._ratfun is None:
+            return self.copy()
         return self.__class__(self._ratfun.canonical(factor_const), **self.assumptions)
 
     def general(self):
@@ -1333,6 +1474,8 @@ class Expr(ExprPrint, ExprMisc):
 
         See also canonical, partfrac, standard, timeconst, and ZPK."""
 
+        if self._ratfun is None:
+            return self.copy()
         return self.__class__(self._ratfun.general(), **self.assumptions)
 
     def partfrac(self, combine_conjugates=False, damping=None):
@@ -1345,6 +1488,8 @@ class Expr(ExprPrint, ExprMisc):
 
         See also canonical, standard, general, timeconst, and ZPK."""
 
+        if self._ratfun is None:
+            return self.copy()        
         return self.__class__(self._ratfun.partfrac(combine_conjugates,
                                                     damping),
                               **self.assumptions)
@@ -1352,19 +1497,25 @@ class Expr(ExprPrint, ExprMisc):
     def recippartfrac(self, combine_conjugates=False, damping=None):
         """Convert rational function into partial fraction form
         using reciprocal of variable.
-        For example,
+
+        For example, if H = 5 * (s**2 + 1) / (s**2 + 5*s + 4)     
+        then H.recippartfrac() gives 
+        5/4 - 10/(3*(1 + 1/s)) + 85/(48*(1/4 + 1/s))
 
         If combine_conjugates is True then the pair of partial
         fractions for complex conjugate poles are combined.
 
         See also canonical, standard, general, partfrac, timeconst, and ZPK."""
 
+        if self._ratfun is None:
+            return self.copy()
+        
         tmpsym = symsymbol('qtmp')
 
-        expr = self.subs(tmpsym)
-        ratfun = Ratfun(expr, tmpsym)
+        expr = self.subs(1 / tmpsym)
+        ratfun = Ratfun(expr.expr, tmpsym)
 
-        nexpr = expr.partfrac(combine_conjugates, damping)
+        nexpr = ratfun.partfrac(combine_conjugates, damping)
         nexpr = nexpr.subs(tmpsym, 1 / self.var)
         
         return self.__class__(nexpr, **self.assumptions)
@@ -1380,7 +1531,8 @@ class Expr(ExprPrint, ExprMisc):
         See also canonical, general, partfrac, timeconst, and ZPK.
 
         """
-
+        if self._ratfun is None:
+            return self.copy()        
         return self.__class__(self._ratfun.standard(), **self.assumptions)
 
     def mixedfrac(self):
@@ -1395,6 +1547,8 @@ class Expr(ExprPrint, ExprMisc):
 
         See also canonical, general, standard, partfrac and ZPK."""
 
+        if self._ratfun is None:
+            return self.copy()        
         return self.__class__(self._ratfun.timeconst(), **self.assumptions)
 
     def ZPK(self):
@@ -1406,6 +1560,8 @@ class Expr(ExprPrint, ExprMisc):
 
         """
 
+        if self._ratfun is None:
+            return self.copy()        
         return self.__class__(self._ratfun.ZPK(), **self.assumptions)
 
     def factored(self):
@@ -1417,7 +1573,9 @@ class Expr(ExprPrint, ExprMisc):
         standard, partfrac, and timeconst.
 
         """
-
+        
+        if self._ratfun is None:
+            return self.copy()
         return self.__class__(self._ratfun.ZPK(), **self.assumptions)
     
     def expandcanonical(self):
@@ -1428,6 +1586,8 @@ class Expr(ExprPrint, ExprMisc):
 
         See also canonical, general, partfrac, timeconst, and ZPK."""
 
+        if self._ratfun is None:
+            return self.copy()        
         return self.__class__(self._ratfun.expandcanonical(), **self.assumptions)
 
     def coeffs(self, norm=False):
@@ -1438,6 +1598,9 @@ class Expr(ExprPrint, ExprMisc):
         
         If norm is True, normalise coefficients to highest power is 1."""
 
+        if self._ratfun is None:
+            return expr([])
+        
         try:
             z = sym.Poly(self.expr, self.var)
         except:
@@ -1464,6 +1627,9 @@ class Expr(ExprPrint, ExprMisc):
 
         This the maximum of the numerator and denominator degrees.
         Note zero has a degree of -inf."""
+
+        if self._ratfun is None:
+            return 1
         
         return self._ratfun.degree
 
@@ -1476,6 +1642,9 @@ class Expr(ExprPrint, ExprMisc):
         Note zero has a degree of -inf.
 
         """
+
+        if self._ratfun is None:
+            return 1
         
         return self._ratfun.Ndegree
 
@@ -1486,18 +1655,12 @@ class Expr(ExprPrint, ExprMisc):
         rational function.
 
         Note zero has a degree of -inf."""
+
+        if self._ratfun is None:
+            return 1        
         
         return self._ratfun.Ddegree
 
-    @property
-    def strictly_proper(self):
-        """Return True if the degree of the dominator is greater
-        than the degree of the numerator.
-        This will throw an exception if the expression is not a
-        rational function."""
-
-        return self._ratfun.strictly_proper
-    
     def prune_HOT(self, degree):
         """Prune higher order terms if expression is a polynomial
         so that resultant approximate expression has the desired degree."""
@@ -1515,7 +1678,82 @@ class Expr(ExprPrint, ExprMisc):
             expr += term
 
         return self.__class__(expr, **self.assumptions)            
+
+    def ratfloat(self):
+        """This converts rational numbers in an expression to floats.
+        See also floatrat.
+
+        For example, t / 5 -> 0.2 * t
+        """
+
+        expr = self.expr
+        expr = expr.replace(lambda expr: expr.is_Rational,
+                            lambda expr: sym.Float(expr))
+
+        return self.__class__(expr, **self.assumptions)
+
+    def floatrat(self):
+        """This converts floating point numbers to rational numbers in an
+        expression.  See also ratfloat.
+
+        For example, 0.2 * t - > t / 5
+
+        """
+
+        expr = self.expr
+        # FIXME...
+        expr = expr.replace(lambda expr: expr.is_Float,
+                            lambda expr: sym.Rational(expr))
+
+        return self.__class__(expr, **self.assumptions)            
     
+    def approximate_fractional_power(self, order=2):
+        """This is an experimental method to approximate
+        s**a, where a is fractional, with a rational function using
+        a Pade approximant."""
+
+        v = self.var
+        
+        def query(expr):
+
+            if not expr.is_Pow:
+                return False
+            if expr.args[0] != v:
+                return False
+            if expr.args[1].is_Number and not expr.args[1].is_Integer:
+                return True
+            if expr.args[1].is_Symbol and not expr.args[1].is_Integer:
+                return True
+            return False
+
+        def value1(expr):
+
+            a = expr.args[1]
+
+            n = v * (a + 1) + (1 - a)
+            d = v * (a - 1) + (1 + a)
+            return n / d
+
+        def value2(expr):
+
+            a = expr.args[1]
+
+            n = v**2 * (a**2 + 3 * a + 2) + v * (8 - a**2) + (a**2 - 3 * a + 2)
+            d = v**2 * (a**2 - 3 * a + 2) + v * (8 - a**2) + (a**2 + 3 * a + 2)
+            return n / d        
+
+        if order == 1:
+            value = value1
+        elif order == 2:
+            value = value2
+        else:
+            raise ValueError('Can only handle order 1 and 2 at the moment')
+        
+        expr = self.expr
+        expr = expr.replace(query, value)
+
+        return self.__class__(expr, **self.assumptions)
+
     
 def expr(arg, **assumptions):
     """Create Lcapy expression from arg.
@@ -1573,6 +1811,8 @@ from .sexpr import Hs, Is, Vs, Ys, Zs, sExpr, sexpr
 from .texpr import tExpr, texpr
 from .impedance import Impedance
 from .admittance import Admittance
+from .voltage import Voltage
+from .current import Current
 from .omegaexpr import Homega, Iomega, Vomega, Yomega, Zomega, omegaExpr, omegaexpr
 
 # Horrible hack to work with IPython around Sympy's back for LaTeX
