@@ -1,4 +1,4 @@
-"""This file provides the fExpr class to represent f-domain (Fourier
+"""This module provides the fExpr class to represent f-domain (Fourier
 domain) expressions.
 
 Copyright 2014--2019 Michael Hayes, UCECE
@@ -42,6 +42,11 @@ class fExpr(Expr):
             result = tExpr(result)
         return result
 
+    def IFT(self, **assumptions):
+        """Convert to t-domain.   This is an alias for inverse_fourier."""
+
+        return self.inverse_fourier(**assumptions)    
+    
     def time(self, **assumptions):
         return self.inverse_fourier()
     
@@ -129,6 +134,7 @@ class Vf(fExpr):
 
     quantity = 'Voltage spectrum'
     units = 'V/Hz'
+    superkind = 'Voltage'        
 
     def __init__(self, val, **assumptions):
 
@@ -142,6 +148,7 @@ class If(fExpr):
 
     quantity = 'Current spectrum'
     units = 'A/Hz'
+    superkind = 'Current'    
 
     def __init__(self, val, **assumptions):
 
