@@ -7,6 +7,7 @@ Copyright 2015--2019 Michael Hayes, UCECE
 """
 
 from __future__ import print_function
+from .expr import expr
 from .cexpr import cExpr
 from .omegaexpr import omegaExpr
 from .symbols import j, omega, jomega, s, t
@@ -22,16 +23,10 @@ import sympy as sym
 
 module = sys.modules[__name__]
 
-# TODO: ponder use of anonymous names when manipulating nets with select, zero, etc.
-
-
-def arg_format(value):
-    """Place value string inside curly braces if it contains a delimiter."""
-    string = str(value)
-    for delimiter in delimiters:
-        if delimiter in string:
-            return '{' + string + '}'
-    return string
+cptaliases = {'E': 'VCVS', 'F': 'CCCS',
+              'G': 'VCCS',  'H': 'CCVS',
+              'r': 'Damper', 'm': 'Mass',
+              'k': 'Spring'}
 
 
 class Cpt(ImmitanceMixin):
@@ -99,8 +94,7 @@ class Cpt(ImmitanceMixin):
         classname = self.classname
         # Handle aliases.
         try:
-            classname = {'E' : 'VCVS', 'F' : 'CCCS',
-                         'G' : 'VCCS',  'H' : 'CCVS'}[classname]
+            classname = cptaliases[classname]
         except:
             pass
                 
@@ -120,20 +114,51 @@ class Cpt(ImmitanceMixin):
     def __str__(self):
         return self.string
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         raise NotImplementedError('stamp method not implemented for %s' % self)
 
-    def copy(self):
+    def _copy(self):
         """Make copy of net."""
         
         return str(self)
     
-    def kill(self):
+    def _arg_format(self, value):
+        """Place value string inside curly braces if it contains a delimiter."""
+
+        string = str(value)
+
+        if string.startswith('{'):
+            return string
+        
+        for delimiter in delimiters:
+            if delimiter in string:
+                return '{' + string + '}'
+        return string
+
+    def _kill(self):
         """Kill sources."""
 
-        return self.copy()
+        return self._copy()
 
-    def netmake(self, node_map=None, zero=False):
+    def _subs(self, subs_dict):
+        """Substitute values using dictionary of substitutions.
+        If a scalar is passed, this is substituted for the component value.
+
+        For example, given a component, cpt, defined as 'R1 1 2' then
+        cpt.subs(5) and cpt.subs({'R1': 5}) are equivalent.  In both
+        cases, the result is 'R1 1 2 5'."""
+
+        if not isinstance(subs_dict, dict):
+            subs_dict = {self.args[0]: subs_dict}
+
+        return self._netsubs(subs_dict=subs_dict)
+
+    def _initialize(self, ic):
+        """Change initial condition to ic."""
+
+        return self._copy()    
+
+    def _netsubs(self, node_map=None, zero=False, subs_dict=None):
         """Create a new net description.  If node_map is not None,
         rename the nodes.  If zero is True, set args to zero."""
 
@@ -147,28 +172,85 @@ class Cpt(ImmitanceMixin):
             field += 1
             if field == self.keyword[0]:
                 string += ' ' + self.keyword[1]
-                field += 1                
-        for arg in self.explicit_args:
+                field += 1
+
+        if subs_dict is None:
+            args = self.explicit_args
+        else:
+            args = self.args
+                
+        for arg in self.args:
             if zero:
                 arg = 0
-            string += ' ' + arg_format(arg)
+
+            # Perform substitutions
+            if subs_dict is not None:
+                arg = str(expr(arg).subs(subs_dict))
+                
+            string += ' ' + self._arg_format(arg)
             field += 1
             if field == self.keyword[0]:
-                string += self.keyword[1]            
-        string += '; %s' % self.opts
+                string += self.keyword[1]
+        opts_str = str(self.opts).strip()
+        if opts_str != '':
+            string += '; ' + opts_str
         return string
         
-    def rename_nodes(self, node_map):
+    def _rename_nodes(self, node_map):
         """Rename the nodes using dictionary node_map."""
 
-        return self.netmake(node_map)
+        return self._netsubs(node_map)
 
-    def select(self, kind=None):
+    def _netmake1(self, name, nodes=None, args=None, opts=None):
+
+        if nodes is None:
+            nodes = self.relnodes
+
+        if args is None:
+            args = self.args
+
+        if not isinstance(args, tuple):
+            args = (args, )
+
+        if opts is None:
+            opts = self.opts
+            
+        fmtargs = []
+        for arg in args:
+            fmtargs.append(self._arg_format(arg))
+
+        if len(fmtargs) == 1 and fmtargs[0] == name:
+            fmtargs = []
+
+        parts = [name]
+        parts.extend(nodes)
+        parts.extend(fmtargs)
+        
+        net = ' '.join(parts)
+        opts_str = str(opts).strip()
+        if opts_str != '':
+            net += '; ' + opts_str
+        return net
+    
+    def _netmake(self, nodes=None, args=None, opts=None):
+        """This keeps the same cpt name"""                
+        return self._netmake1(self.namespace + self.relname, nodes, args, opts)
+
+    def _netmake_anon(self, newtype, nodes=None, opts=None):
+        """This is used for changing cpt name from C1 to O"""        
+        return self._netmake1(self.namespace + newtype, nodes, args=(),
+                              opts=opts)
+        
+    def _netmake_variant(self, newtype, nodes=None, args=None, opts=None):
+        """This is used for changing cpt name from C1 to ZC1"""
+        return self._netmake1(self.namespace + newtype + self.relname,
+                              nodes, args, opts)
+    
+    def _select(self, kind=None):
         """Select domain kind for component."""
-
         raise ValueError('Component not a source: %s' % self)    
 
-    def zero(self):
+    def _zero(self):
         """Zero value of the voltage source.  This kills it but keeps it as a
         voltage source in the netlist.  This is required for dummy
         voltage sources that are required to specify the controlling
@@ -176,20 +258,17 @@ class Cpt(ImmitanceMixin):
 
         raise ValueError('Component not a source: %s' % self)        
 
-    def s_model(self, var):
+    def _s_model(self, var):
         """Return s-domain model of component."""
+        return self._copy()
 
-        return self.copy()
-
-    def ss_model(self):
+    def _ss_model(self):
         """Return state-space model of component."""
+        return self._copy()            
 
-        return self.copy()            
-
-    def pre_initial_model(self):
+    def _pre_initial_model(self):
         """Return pre-initial model of component."""
-
-        return self.copy()        
+        return self._copy()        
 
     @property
     def is_source(self):
@@ -452,25 +531,25 @@ class Invalid(Cpt):
 
 class NonLinear(Invalid):
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         raise NotImplementedError('Cannot analyse non-linear component: %s' % self)
 
 
 class TimeVarying(Invalid):
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         raise NotImplementedError('Cannot analyse time-varying component: %s' % self)
 
 
 class Logic(Invalid):
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         raise NotImplementedError('Cannot analyse logic component: %s' % self)
 
 
 class Misc(Invalid):
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         raise NotImplementedError('Cannot analyse misc component: %s' % self)
 
 
@@ -487,15 +566,18 @@ class Dummy(Cpt):
 class XX(Dummy):
     directive = True
     
-    def stamp(self, cct):
+    def _stamp(self, cct):
         pass
 
+    def _subs(self, subs_dict):
+        return self._copy()
+    
     
 class IndependentSource(Cpt):
 
     independent_source = True
     
-    def zero(self):
+    def _zero(self):
         """Zero value of the source.  For a voltage source this makes it a
         short-circuit; for a current source this makes it
         open-circuit.  This effectively kills the source but keeps it
@@ -504,25 +586,23 @@ class IndependentSource(Cpt):
         current for CCVS and CCCS components.
 
         """
-        return self.netmake(zero=True)
+        return self._netsubs(zero=True)
+    
 
 class DependentSource(Dummy):
 
     dependent_source = True        
 
-    def zero(self):
-        return self.copy()
+    def _zero(self):
+        return self._copy()
 
     
 class RLC(Cpt):
 
-    def s_model(self, var):
+    def _s_model(self, var):
 
-        if self.Voc == 0:        
-            return '%sZ%s %s %s %s; %s' % (self.namespace, self.relname, 
-                                           self.relnodes[0], self.relnodes[1],
-                                           arg_format(self.Z(var)), 
-                                           self.opts)
+        if self.Voc == 0:
+            self._netmake_variant('Z', args=self.Z(var))
 
         dummy_node = self.dummy_node()
 
@@ -532,19 +612,14 @@ class RLC(Cpt):
         # in parallel with Z and V.
         voltage_opts = opts.strip_voltage_labels()
 
-        znet = '%sZ%s %s %s %s; %s' % (self.namespace, self.relname, 
-                                       self.relnodes[0], dummy_node,
-                                       arg_format(self.Z(var)), 
-                                       opts)
+        znet = self._netmake_variant('Z', nodes=(self.relnodes[0], dummy_node),
+                                     args=self.Z(var), opts=opts)
 
         # Strip voltage and current labels from voltage source.
         opts.strip_all_labels()
 
-        vnet = '%sV%s %s %s s %s; %s' % (self.namespace, self.relname, 
-                                         dummy_node, self.relnodes[1],
-                                         arg_format(self.Voc.laplace()(var)),
-                                         opts)
-
+        vnet = self._netmake_variant('V', nodes=(dummy_node, self.relnodes[1]),
+                                     args=self.Voc.laplace()(var), opts=opts)
         if voltage_opts == {}:
             return znet + '\n' + vnet
 
@@ -558,32 +633,28 @@ class RLC(Cpt):
         # v_L(t) -> V_L(s)
         for opt, val in voltage_opts.items():
             opts[opt] = capitalize_name(val)
-            
-        onet = '%sO%s %s %s; %s' % (self.namespace, self.relname, 
-                                    self.relnodes[0], self.relnodes[1], opts)
+
+        onet = self._netmake_variant('O', opts=opts)
         return znet + '\n' + vnet + '\n' + onet
 
 
 class RC(RLC):
 
-    def noisy(self):
+    def _noisy(self):
 
         dummy_node = self.dummy_node()
 
         opts = self.opts.copy()
 
-        rnet = '%s %s %s %s; %s' % (self.defname, 
-                                    self.relnodes[0], dummy_node,
-                                    arg_format(self.args[0]),
-                                    opts)
+        rnet = self._netmake(nodes=(self.relnodes[0], dummy_node),
+                             opts=opts)                                  
         
         Vn = 'sqrt(4 * k * T * %s)' % self.args[0]
-        vnet = '%sVn%s %s %s noise %s; %s' % (
-            self.namespace, self.relname, dummy_node,
-            self.relnodes[1], arg_format(Vn), opts)
+        vnet = self._netmake_variant('Vn', nodes=(dummy_node, self.relnodes[1]),
+                                     args=('noise', Vn), opts=opts)
         return rnet + '\n' + vnet
     
-    def stamp(self, cct):
+    def _stamp(self, cct):
 
         # L's can also be added with this stamp but if have coupling
         # it is easier to generate a stamp that requires the branch current
@@ -619,27 +690,24 @@ class C(RC):
     def C(self):
         return self.cpt.C
     
-    def kill(self):
+    def _kill(self):
         """Kill implicit sources due to initial conditions."""
-        return '%s %s %s %s; %s' % (
-            self.defname, self.relnodes[0], self.relnodes[1],
-            arg_format(self.args[0]), self.opts)
+        return self.netmake(args=self.args[0])
 
-    def pre_initial_model(self):
+    def _initialize(self, ic):
+        """Change initial condition to ic."""
+        return self._netmake(args=(self.args[0], ic))
+
+    def _pre_initial_model(self):
 
         if self.cpt.v0 == 0.0:
-            return '%sO %s %s; %s' % (self.namespace, self.relnodes[0],
-                                      self.relnodes[1], self.opts)
-        return '%sV%s %s %s %s; %s' % (self.namespace, self.relname,
-                                       self.relnodes[0], self.relnodes[1], 
-                                       arg_format(self.cpt.v0), self.opts)
+            return self._netmake_anon('O')
+        return self._netmake_variant('V', args=self.cpt.v0)
 
-    def ss_model(self):
+    def _ss_model(self):
         # Perhaps mangle name to ensure it does not conflict
         # with another voltage source?
-        return '%sV_%s %s %s; %s' % (self.namespace, self.relname,
-                                     self.relnodes[0], self.relnodes[1], 
-                                     self.opts)
+        return self._netmake_variant('V_', args=())
         
     @property
     def V0(self):
@@ -663,7 +731,7 @@ class VCVS(DependentSource):
 
     need_branch_current = True
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         n1, n2, n3, n4 = self.node_indexes
         m = self.branch_index
 
@@ -681,20 +749,20 @@ class VCVS(DependentSource):
         if n4 >= 0:
             cct._C[m, n4] += A
 
-    def kill(self):
+    def _kill(self):
         newopts = self.opts.copy()
         newopts.strip_current_labels()
         newopts.strip_labels()
 
-        return '%sW %s %s; %s' % (self.namespace, self.relnodes[0],
-                                  self.relnodes[1], newopts)
-            
+        return self._netmake_anon('W', opts=newopts)
+
+    
 class CCCS(DependentSource):
     """CCCS"""
 
     need_control_current = True
     
-    def stamp(self, cct):
+    def _stamp(self, cct):
         n1, n2 = self.node_indexes
         m = cct._branch_index(self.args[0])
         F = cExpr(self.args[1]).expr
@@ -704,13 +772,13 @@ class CCCS(DependentSource):
         if n2 >= 0:
             cct._B[n2, m] += F
 
-    def kill(self):
+    def _kill(self):
         newopts = self.opts.copy()
         newopts.strip_voltage_labels()
         newopts.strip_labels()
 
-        return '%sO %s %s; %s' % (self.namespace, self.relnodes[0],
-                                  self.relnodes[1], newopts)
+        return self._netmake_anon('O', opts=newopts)
+
 
 class FB(Misc):
     """Ferrite bead"""
@@ -720,7 +788,7 @@ class FB(Misc):
 class VCCS(DependentSource):
     """VCCS"""
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         n1, n2, n3, n4 = self.node_indexes
         G = cExpr(self.args[0]).expr
 
@@ -733,13 +801,12 @@ class VCCS(DependentSource):
         if n2 >= 0 and n4 >= 0:
             cct._G[n2, n4] -= G
 
-    def kill(self):
+    def _kill(self):
         newopts = self.opts.copy()
         newopts.strip_voltage_labels()
         newopts.strip_labels()
 
-        return '%sO %s %s; %s' % (self.namespace, self.relnodes[0],
-                                  self.relnodes[1], newopts)
+        return self._netmake_anon('O', opts=newopts)        
 
 class GY(Dummy):
     """Gyrator"""    
@@ -747,7 +814,7 @@ class GY(Dummy):
     need_branch_current = True
     need_extra_branch_current = True
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         
         n1, n2, n3, n4 = self.node_indexes
         m1 = self.cct._branch_index(self.defname + 'X')
@@ -790,7 +857,7 @@ class CCVS(DependentSource):
     need_branch_current = True
     need_control_current = True
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         n1, n2 = self.node_indexes
         m = self.branch_index
 
@@ -805,32 +872,28 @@ class CCVS(DependentSource):
         G = cExpr(self.args[1]).expr
         cct._D[m, mc] -= G
 
-    def kill(self):
+    def _kill(self):
         newopts = self.opts.copy()
         newopts.strip_current_labels()
         newopts.strip_labels()
 
-        return '%sW %s %s; %s' % (self.namespace, self.relnodes[0],
-                                  self.relnodes[1], newopts)        
+        return self._netmake_anon('O', opts=newopts)                
+
 
 class I(IndependentSource):
 
-    def select(self, kind=None):
+    def _select(self, kind=None):
         """Select domain kind for component."""
+        return self._netmake(args=self.cpt.Isc.netval(kind))
 
-        return '%s %s %s %s; %s' % (
-            self.defname, self.relnodes[0], self.relnodes[1],
-            self.cpt.Isc.netval(kind), self.opts)
-
-    def kill(self):
+    def _kill(self):
         newopts = self.opts.copy()
         newopts.strip_voltage_labels()
         newopts.strip_labels()
 
-        return '%sO %s %s; %s' % (self.namespace, self.relnodes[0],
-                                  self.relnodes[1], newopts)
+        return self._netmake_anon('O', opts=newopts)                        
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
 
         n1, n2 = self.node_indexes
 
@@ -841,24 +904,16 @@ class I(IndependentSource):
         if n2 >= 0:
             cct._Is[n2] -= I
 
-    def ss_model(self):
-        return '%s%s %s %s {%s(t)}; %s' % (self.namespace, self.relname,
-                                           self.relnodes[0], self.relnodes[1],
-                                           self.relname.lower(),
-                                           self.opts)
+    def _ss_model(self):
+        return self._netmake(args='%s(t)' % self.relname.lower)
 
-    def s_model(self, var):
+    def _s_model(self, var):
+        return self._netmake(args=self.Isc.laplace()(var))
 
-        return '%s %s %s s %s; %s' % (self.defname, 
-                                      self.nodes[0], self.nodes[1],
-                                      arg_format(self.Isc.laplace()(var)),
-                                      self.opts)
-
-    def pre_initial_model(self):
+    def _pre_initial_model(self):
 
         # Assume IC zero.  FIXME
-        return '%sO %s %s; %s' % (self.namespace, self.relnodes[0],
-                                  self.relnodes[1], self.opts)
+        return self._netmake_anon('O', opts=newopts)
 
 
 class K(Dummy):
@@ -872,7 +927,7 @@ class K(Dummy):
                                  cpt_type, cpt_id, string,
                                  opts_string, nodes, keyword, *args)
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
 
         if cct.kind == 'dc':
             return
@@ -913,13 +968,15 @@ class L(RLC):
     def L(self):
         return self.cpt.L
     
-    def kill(self):
+    def _kill(self):
         """Kill implicit sources due to initial conditions."""
-        return '%s %s %s %s; %s' % (
-            self.defname, self.relnodes[0], self.relnodes[1],
-            arg_format(self.args[0]), self.opts)
+        return self.netmake(args=self.args[0])        
 
-    def stamp(self, cct):
+    def _initialize(self, ic):
+        """Change initial condition to ic."""
+        return self._netmake(args=(self.args[0], ic))
+
+    def _stamp(self, cct):
 
         # This formulation adds the inductor current to the unknowns
 
@@ -944,26 +1001,21 @@ class L(RLC):
             V = self.Voc.expr            
             cct._Es[m] += V
 
-    def ss_model(self):
+    def _ss_model(self):
         # Perhaps mangle name to ensure it does not conflict
-        # with another current source?        
-        return '%sI_%s %s %s {-i_%s(t)}; %s' % (self.namespace, self.relname,
-                                                self.relnodes[0], self.relnodes[1],
-                                                self.relname, self.opts)
+        # with another current source?
+        return self._netmake_variant('I_', args='-i_%s(t)' % self.relname)
     
-    def pre_initial_model(self):
+    def _pre_initial_model(self):
 
         if self.cpt.i0 == 0.0:
-            return '%sW %s %s; %s' % (self.namespace, self.relnodes[0],
-                                      self.relnodes[1], self.opts)
-        return '%sI%s %s %s %s; %s' % (self.namespace, self.relname,
-                                       self.relnodes[0], self.relnodes[1], 
-                                       arg_format(self.cpt.i0), self.opts)
+            return self._netmake_anon('W')                    
+        return self._netmake_variant('I', args=self.cpt.i0)
 
 class O(Dummy):
     """Open circuit"""
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         pass
 
     @property
@@ -994,7 +1046,7 @@ class SPpp(Dummy):
 
     need_branch_current = True
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         n1, n2, n3 = self.node_indexes
         m = self.branch_index
 
@@ -1012,7 +1064,7 @@ class SPpm(Dummy):
 
     need_branch_current = True
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         n1, n2, n3 = self.node_indexes
         m = self.branch_index
 
@@ -1029,7 +1081,7 @@ class SPppp(Dummy):
 
     need_branch_current = True
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         n1, n2, n3, n4 = self.node_indexes
         m = self.branch_index
 
@@ -1048,7 +1100,7 @@ class SPpmm(Dummy):
 
     need_branch_current = True
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         n1, n2, n3, n4 = self.node_indexes
         m = self.branch_index
 
@@ -1068,7 +1120,7 @@ class SPppm(Dummy):
 
     need_branch_current = True
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         n1, n2, n3, n4 = self.node_indexes
         m = self.branch_index
 
@@ -1089,7 +1141,7 @@ class TF(Cpt):
 
     need_branch_current = True
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
 
         n1, n2, n3, n4 = self.node_indexes
         m = self.branch_index
@@ -1101,7 +1153,7 @@ class TF(Cpt):
             cct._B[n2, m] -= 1
             cct._C[m, n2] -= 1
         
-        T = self.cpt.alpha
+        T = self.cpt.alpha.expr
 
         if n3 >= 0:
             cct._B[n3, m] -= T
@@ -1114,7 +1166,7 @@ class TF(Cpt):
 class TFtap(Cpt):
     """Tapped transformer"""    
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         raise NotImplementedError('Cannot analyse tapped transformer %s' % self)
 
 
@@ -1144,7 +1196,7 @@ class TR(Dummy):
 
     need_branch_current = True
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         n1, n2 = self.node_indexes
         m = self.branch_index
 
@@ -1163,22 +1215,18 @@ class V(IndependentSource):
     need_branch_current = True
     flip_branch_current = True        
 
-    def select(self, kind=None):
+    def _select(self, kind=None):
         """Select domain kind for component."""
+        return self._netmake(args=self.cpt.Voc.netval(kind))
 
-        return '%s %s %s %s; %s' % (
-            self.defname, self.relnodes[0], self.relnodes[1],
-            self.cpt.Voc.netval(kind), self.opts)        
-
-    def kill(self):
+    def _kill(self):
         newopts = self.opts.copy()
         newopts.strip_current_labels()
         newopts.strip_labels()
 
-        return '%sW %s %s; %s' % (self.namespace, self.relnodes[0],
-                                  self.relnodes[1], newopts)
+        return self._netmake_anon('W', opts=newopts)                            
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
 
         n1, n2 = self.node_indexes
         m = self.branch_index
@@ -1193,30 +1241,22 @@ class V(IndependentSource):
         V = self.Voc.expr
         cct._Es[m] += V
 
-    def ss_model(self):
-        return '%s%s %s %s {%s(t)}; %s' % (self.namespace, self.relname,
-                                           self.relnodes[0], self.relnodes[1],
-                                           self.relname.lower(),
-                                           self.opts)
+    def _ss_model(self):
+        return self._netmake(args='%s(t)' % self.relname.lower())
 
-    def s_model(self, var):
+    def _s_model(self, var):
+        return self._netmake(args=self.cpt.Voc.laplace()(var))
 
-        return '%s %s %s s %s; %s' % (self.defname, 
-                                      self.relnodes[0], self.relnodes[1],
-                                      arg_format(self.cpt.Voc.laplace()(var)),
-                                      self.opts)
-
-    def pre_initial_model(self):
+    def _pre_initial_model(self):
 
         # Assume IC zero.  FIXME
-        return '%sW %s %s; %s' % (self.namespace, self.relnodes[0],
-                                  self.relnodes[1], self.opts)
+        return self._netmake_anon('W')
 
 
 class W(Dummy):
     """Wire"""
 
-    def stamp(self, cct):
+    def _stamp(self, cct):
         pass
 
     @property
@@ -1266,7 +1306,6 @@ def make(classname, parent, name, cpt_type, cpt_id,
 
     cpt = newclass(parent, name, cpt_type, cpt_id, string, opts_string, 
                    nodes, *args)
-    # Add named attributes for the args?   Lname1, etc.
 
     return cpt
 
@@ -1289,6 +1328,7 @@ defcpt('Dzener', 'D', 'Zener diode')
 defcpt('E', VCVS, 'VCVS')
 defcpt('Eopamp', VCVS, 'Opamp')
 defcpt('Efdopamp', VCVS, 'Fully differential opamp')
+defcpt('Eamp', VCVS, 'Amplifier')
 
 defcpt('F', CCCS, 'CCCS')
 defcpt('FS', Misc, 'Fuse')
@@ -1311,6 +1351,7 @@ defcpt('Jpjf', 'J', 'P JFET transistor')
 defcpt('M', NonLinear, 'N MOSJFET transistor')
 defcpt('Mnmos', 'M', 'N channel MOSJFET transistor')
 defcpt('Mpmos', 'M', 'P channel MOSJFET transistor')
+defcpt('MISC', Misc, 'Generic circuitikz bipole')
 defcpt('MT', Misc, 'Motor')
 defcpt('MX', Misc, 'Mixer')
 
@@ -1335,7 +1376,11 @@ defcpt('Ubuffer', Logic, 'Buffer')
 defcpt('Upbuffer', Logic, 'Buffer with power supplies')
 defcpt('Uinverter', Logic, 'Inverter')
 defcpt('Upinverter', Logic, 'Inverter with power supplies')
+defcpt('Uinamp', Misc, 'Instrumentation amplifier')
+defcpt('Uisoamp', Misc, 'Isolated amplifier')
 defcpt('Udiffamp', Misc, 'Differential amplifier')
+defcpt('Ufdopamp', Misc, 'Fully differential opamp ')
+defcpt('Uopamp', Misc, 'Opamp ')
 defcpt('Uregulator', Misc, 'Regulator ')
 defcpt('Uadc', Misc, 'ADC')
 defcpt('Udac', Misc, 'DAC')
@@ -1354,6 +1399,7 @@ defcpt('Uchip4444', Logic, 'General purpose chip')
 defcpt('Uchip8181', Logic, 'General purpose chip')
 defcpt('Uchip8888', Logic, 'General purpose chip')
 defcpt('Umux21', Logic, '2-1 multiplexer')
+defcpt('Umux41', Logic, '4-1 multiplexer')
 defcpt('Umux42', Logic, '4-2 multiplexer')
 defcpt('Udff', Misc, 'D flip-flop')
 defcpt('Ujkff', Misc, 'JK flip-flop')
@@ -1367,6 +1413,13 @@ defcpt('Vac', V, 'AC voltage source')
 defcpt('Vnoise', V, 'Noise voltage source')
 defcpt('VM', O, 'Voltmeter')
 
+# Let's choose mechanical analogue II (the impedance analogue) where
+# force is equivalent to voltage and velocity is equivalent to
+# current.  With this analogy parallel and serial have to be switched.
+defcpt('m', L, 'Mass')
+defcpt('k', C, 'Spring')
+defcpt('r', R, 'Damper')
+
 # Append classes defined in this module but not imported.
 clsmembers = inspect.getmembers(module, lambda member: inspect.isclass(member) and member.__module__ == __name__)
 for name, cls in clsmembers:
