@@ -212,7 +212,7 @@ class NetlistNamespace(object):
 
         cct = self
         if kwargs.pop('s_model', False):
-            cct = cct.s_model()
+            cct = cct._s_model()
 
         return cct.sch.draw(filename=filename, **kwargs)
 
@@ -269,7 +269,6 @@ class NetlistMixin(object):
         
         return self.netlist()
 
-    @property    
     def pdb(self):
         """Enter the python debugger."""
         
@@ -357,7 +356,7 @@ class NetlistMixin(object):
         new = self._new()
 
         for cpt in self._elements.values():
-            new._add(cpt.copy())
+            new._add(cpt._copy())
         return new        
 
     def _new(self):
@@ -506,7 +505,7 @@ class NetlistMixin(object):
         new = self._new()
 
         for cpt in self._elements.values():
-            new._add(cpt.rename_nodes(node_map))
+            new._add(cpt._rename_nodes(node_map))
         return new                
 
     @property
@@ -767,9 +766,9 @@ class NetlistMixin(object):
 
         for cpt in self._elements.values():
             if cpt.independent_source:
-                net = cpt.select(kind)                
+                net = cpt._select(kind)                
             else:
-                net = cpt.copy()
+                net = cpt._copy()
             new._add(net)
         return new        
 
@@ -780,11 +779,11 @@ class NetlistMixin(object):
         for cpt in self._elements.values():
             if cpt.name in sourcenames:
                 if cpt.name in self.control_sources:
-                    net = cpt.zero()                
+                    net = cpt._zero()                
                 else:
-                    net = cpt.kill()
+                    net = cpt._kill()
             else:
-                net = cpt.copy()                
+                net = cpt._copy()                
             new._add(net)
         return new        
 
@@ -837,9 +836,9 @@ class NetlistMixin(object):
 
         for cpt in self._elements.values():
             if cpt.name in resistornames:
-                net = cpt.noisy()
+                net = cpt._noisy()
             else:
-                net = cpt.copy()
+                net = cpt._copy()
             new._add(net)
         return new        
 
@@ -913,6 +912,34 @@ class NetlistMixin(object):
 
         self._ss = StateSpace(self)
         return self._ss
+
+    def subs(self, subs_dict):
+        """Substitute values using dictionary of subsitutions."""        
+
+        new = self._new()
+
+        for cpt in self._elements.values():
+            net = cpt._subs(subs_dict)
+            new._add(net)
+        return new                        
+    
+    def initialize(self, cct, time):
+        """Set the initial values for this netlist based on the values
+        computed for netlist cct at specified time."""        
+
+        new = self._new()
+
+        for cpt in self._elements.values():
+            ic = 0
+            if cpt.name in cct.reactances:
+                if cpt.type == 'C':
+                    ic = cct[cpt.name].v.remove_condition()(time)
+                else:
+                    ic = cct[cpt.name].i.remove_condition()(time) 
+                    
+            net = cpt._initialize(ic)
+            new._add(net)
+        return new                
     
     def pre_initial_model(self):
         """Generate circuit model for determining the pre-initial
@@ -921,7 +948,7 @@ class NetlistMixin(object):
         new = self._new()
 
         for cpt in self._elements.values():
-            net = cpt.pre_initial_model()
+            net = cpt._pre_initial_model()
             new._add(net)
         return new        
 
@@ -931,7 +958,7 @@ class NetlistMixin(object):
         new = self._new()
 
         for cpt in self._elements.values():
-            net = cpt.s_model(var)
+            net = cpt._s_model(var)
             new._add(net)
         return new
 
@@ -942,7 +969,7 @@ class NetlistMixin(object):
         new = self._new()
 
         for cpt in self._elements.values():
-            net = cpt.ss_model()
+            net = cpt._ss_model()
             new._add(net)
         return new            
 
@@ -1065,6 +1092,12 @@ class NetlistMixin(object):
         implicit sources due to initial conditions)."""
 
         return self.dependent_sources + self.independent_sources
+
+    @property
+    def reactances(self):
+        """Return dictionary of reactances."""
+
+        return self.analysis['reactances']
     
     @property
     def independent_sources(self):
@@ -1142,6 +1175,8 @@ class NetlistMixin(object):
         independent_sources = []
         dependent_sources = []        
         control_sources = []
+        reactances = []
+        
         for key, elt in self.elements.items():
             if elt.need_control_current:
                 control_sources.append(elt.args[0])
@@ -1166,6 +1201,7 @@ class NetlistMixin(object):
                 dependent_sources.append(key)
             if elt.reactive:
                 reactive = True
+                reactances.append(key)                
 
         num_sources = len(independent_sources)
                     
@@ -1177,6 +1213,7 @@ class NetlistMixin(object):
         analysis['has_ac'] = ac_count > 0        
         analysis['has_s'] = has_s
         analysis['has_transient'] = has_transient
+        analysis['reactances'] = reactances 
         analysis['dependent_sources'] = dependent_sources        
         analysis['independent_sources'] = independent_sources
         analysis['control_sources'] = control_sources        
