@@ -10,6 +10,7 @@ from .circuit import Circuit
 from .state import state
 
 class Network(object):
+    """This is the base class for network objects."""
 
     voltage_source = False
     current_source = False
@@ -25,10 +26,14 @@ class Network(object):
     netname = ''
     netkeyword = ''
 
-    def __init__(self):
-
-        super(Network, self).__init__()
+    def _make_id(self, kind):
+        """Make identifier"""
 
+        if kind not in self._anon:
+            self._anon[kind] = 0
+        self._anon[kind] += 1
+        return self._anon[kind]
+        
     def _tweak_args(self):
 
         if not hasattr(self, 'args'):
@@ -118,21 +123,30 @@ class Network(object):
 
         netname = self.__class__.__name__ if self.netname == '' else self.netname
 
+        netid = net._make_id(netname)
         if self.netkeyword != '':
-            return '%s %s %s %s %s; right' % (netname, n1, n2, 
-                                              self.netkeyword, self.netargs())
+            return '%s%s %s %s %s %s; right' % (netname, netid,
+                                                n1, n2, 
+                                                self.netkeyword, self.netargs())
         else:
-            return '%s %s %s %s; right' % (netname, n1, n2, self.netargs())
+            return '%s%s %s %s %s; right' % (netname, netid,
+                                             n1, n2, self.netargs())
 
     def netlist(self):
 
         # Enumerate from node 0
         self.node_counter = 0
+        self._anon = {}
         n1 = self.node
         n2 = self.node        
         return self.net_make(self, n2, n1)
 
-
+    def pdb(self):
+        """Enter the python debugger."""
+        
+        import pdb; pdb.set_trace()
+        return self
+    
     @property
     def sch(self):
         """Convert a Network object into a Schematic object."""
