@@ -1,4 +1,4 @@
-"""This file provides the nExpr class to represent discrete-time expressions.
+"""This module provides the nExpr class to represent discrete-time expressions.
 
 Copyright 2020 Michael Hayes, UCECE
 
@@ -12,7 +12,7 @@ from .dsym import nsym, ksym, zsym, dt
 from .acdc import ACChecker, is_dc, is_ac, is_causal
 from .ztransform import ztransform
 from .sequence import Sequence
-from sympy import Sum, summation, Eq, limit
+from sympy import Sum, summation, limit
 
 
 __all__ = ('Hn', 'In', 'Vn', 'Yn', 'Zn')
@@ -28,7 +28,6 @@ class nExpr(dExpr):
 
     def __init__(self, val, **assumptions):
 
-        assumptions['real'] = True
         super(nExpr, self).__init__(val, **assumptions)
 
         #self._fourier_conjugate_class = fExpr
@@ -36,7 +35,7 @@ class nExpr(dExpr):
 
         if self.expr.find(zsym) != set():
             raise ValueError(
-                'n-domain expression %s cannot depend on s' % self.expr)
+                'n-domain expression %s cannot depend on z' % self.expr)
 
     def infer_assumptions(self):
 
@@ -206,21 +205,16 @@ class nExpr(dExpr):
     
         return self.discrete_time_fourier_transform(**assumptions)
 
-    def difference_equation(self, input='x', output='y'):
+    def difference_equation(self, input='x', output='y', form='iir'):
+        """Create difference equation from impulse response.
 
-        x = nexpr('%s(n)' % input)
-        y = nexpr('%s(n)' % output)
+        form can be 'fir' or 'iir' ('direct form I').
+        """
 
-        X = x.ZT()
-        Y = y.ZT()
-        H = self.ZT().partfrac()
+        H = self.ZT()
+        return H.difference_equation(input, output, form)
 
-        # FIR form
-        lhs = y
-        rhs = (H * X).IZT()        
-        return nExpr(Eq(lhs.expr, rhs.expr))
     
-        
 class Yn(nExpr):
 
     """t-domain 'admittance' value."""
@@ -287,7 +281,7 @@ class Hn(nExpr):
         self._ztransform_conjugate_class = Hz
         self._fourier_conjugate_class = Hk
 
-def nexpr(arg):
+def nexpr(arg, **assumptions):
     """Create nExpr object.  If `arg` is nsym return n"""
 
     if arg is nsym:
@@ -302,8 +296,8 @@ def nexpr(arg):
     
     if isinstance(arg, (list, ndarray)):
         return Sequence(arg, var=n).as_impulses()
-    
-    return nExpr(arg)
+
+    return nExpr(arg, **assumptions)
 
 from .zexpr import Hz, Iz, Vz, Yz, Zz, zExpr
 from .kexpr import Hk, Ik, Vk, Yk, Zk, kExpr
