@@ -1,13 +1,13 @@
 from __future__ import division
 from .sym import symsimplify
 from .functions import sqrt
-from .sym import pi
+from .sym import pi, omegasym, fsym
 from .state import state
-from .omegaexpr import omegaExpr
+from .expr import Expr
 import sympy as sym
 import numpy as np
 
-class noiseExpr(omegaExpr):
+class noiseExpr(Expr):
     """Frequency domain (one-sided) noise spectrum expression (amplitude
     spectral density).
 
@@ -51,12 +51,25 @@ class noiseExpr(omegaExpr):
                 assumptions['nid'] = 'n0'
             else:
                 assumptions['nid'] = self._new_nid()
+
         super(noiseExpr, self).__init__(val, **assumptions)
 
     @property
     def nid(self):
         return self.assumptions['nid']
-        
+
+
+    def subs(self, *args, **kwargs):
+        return super(noiseExpr, self).subs(*args, nid=self.nid, **kwargs)
+
+    def __compat__(self, x):
+
+        if not isinstance(x, noiseExpr):
+            return x
+        if self.var == x.var:
+            return x
+        return x(self.var)
+    
     def __add__(self, x):
         """Add noise spectra (on power basis if uncorrelated)."""
 
@@ -65,6 +78,8 @@ class noiseExpr(omegaExpr):
         
         if x == 0:
             return self.__class__(self, nid=self.nid)
+
+        x = self.__compat__(x)
         
         if self.nid == x.nid:
             return self.__class__(self.expr + x.expr, nid=self.nid)
@@ -84,7 +99,9 @@ class noiseExpr(omegaExpr):
             raise ValueError('Cannot subtract %s and %s' % (self, x))
 
         if x == 0:
-            return self.__class__(self, nid=self.nid)        
+            return self.__class__(self, nid=self.nid)
+
+        x = self.__compat__(x)        
 
         if self.nid == x.nid:
             return self.__class__(self.expr - x.expr, nid=self.nid)        
@@ -96,6 +113,8 @@ class noiseExpr(omegaExpr):
     def __mul__(self, x):
         if isinstance(x, noiseExpr) and self.nid != x.nid:
             raise ValueError('Cannot multiply %s and %s' % (self, x))
+
+        x = self.__compat__(x)
         return self.__class__(self.expr * x, nid=self.nid)
 
     def __rmul__(self, x):
@@ -116,6 +135,7 @@ class noiseExpr(omegaExpr):
         except:
             pass
 
+        x = self.__compat__(x)        
         try:
             if self.expr == x.expr:
                 return True
@@ -129,7 +149,9 @@ class noiseExpr(omegaExpr):
     def rms(self):
         """Calculate rms value."""
 
-        P = sym.integrate(self.expr**2, (self.var, 0, sym.oo)) / (2 * sym.pi)
+        P = sym.integrate(self.expr**2, (self.var, 0, sym.oo))
+        if self.var == omegasym:
+            P /= 2 * sym.pi
         rms = sym.sqrt(P)
         # TODO: Use rms class?
         return self._fourier_conjugate_class(rms)
@@ -140,15 +162,17 @@ class noiseExpr(omegaExpr):
 
         N = len(t)
         if N < 3:
-            raise ValueError('Require at least 3 samples')            
+            raise ValueError('Require at least 3 samples')
+        if N & 1:
+            raise ValueError('Require even number of samples')                    
         
         td = np.diff(t)
         if not np.allclose(np.diff(td), 0):
             raise ValueError('Require uniform sampling')
 
         fs = 1 / td[0]
-        f = np.arange(N // 2 + 1) * fs / N
-        Sn = self.evaluate(f * 2 * np.pi)
+        vf = np.arange(N // 2 + 1) * fs / N
+        Sn = self(f).evaluate(vf)
     
         x = np.random.randn(N)
         X = np.fft.rfft(x)
@@ -173,7 +197,6 @@ class noiseExpr(omegaExpr):
         return S.inverse_fourier()
 
     def plot(self, fvector=None, **kwargs):
-
         """Plot frequency response at values specified by fvector.
 
         There are many plotting options, see matplotlib.pyplot.plot.
@@ -186,56 +209,14 @@ class noiseExpr(omegaExpr):
         By default complex data is plotted as separate plots of magnitude (dB)
         and phase.
         """
+        from .plot import plot_frequency, plot_angular_frequency
 
-        from .plot import plot_frequency
-        from .symbols import omega
-        
-        # Hack so show as linear frequency. 
-        obj = self.subs(omega, 2 * pi * omega)
-        obj.domain_name = 'Frequency'
-        obj.domain_units = 'Hz'
-        if hasattr(self, 'part'):
-            obj.part = self.part
-        return plot_frequency(obj, fvector, **kwargs)    
+        if self.var == fsym:
+            return plot_frequency(self, fvector, **kwargs)
+        elif self.var == omegasym:
+            return plot_angular_frequency(self, fvector, **kwargs)
+        raise ValueError('Invalid var for noiseExpr')
 
-class Vnoisy(noiseExpr):
-    """Voltage noise amplitude spectral density (units V/rtHz).
-    This can be a function of angular frequency, omega.  For example,
-    to model an opamp voltage noise:
-
-    v = Vnoisy(1e-8 / sqrt(omega) + 8e-9)
     
-    """
-
-    quantity = 'Voltage noise spectral density'
-    units = 'V/rtHz'
-
-    def __init__(self, val, **assumptions):
-
-        assumptions['positive'] = True
-        super(Vnoisy, self).__init__(val, **assumptions)
-        # FIXME
-        self._fourier_conjugate_class = Vt
-
-
-class Inoisy(noiseExpr):
-    """Current noise amplitude spectral density (units A/rtHz).
-
-    This can be a function of angular frequency, omega.  For example,
-    to model an opamp current noise:
-
-    i = Inoisy(3e-12 / sqrt(omega) + 200e-15)
-    """
-
-    quantity = 'Current noise spectral density'
-    units = 'A/rtHz'
-
-    def __init__(self, val, **assumptions):
-
-        assumptions['positive'] = True
-        super(Inoisy, self).__init__(val, **assumptions)
-        # FIXME
-        self._fourier_conjugate_class = It
-
-        
-from .texpr import It, Vt        
+from .fexpr import f
+from .omegaexpr import omega
