@@ -1,14 +1,14 @@
-"""This file provides the omegaExpr class to represent omega-domain
+"""This module provides the omegaExpr class to represent omega-domain
 (angular frequency Fourier domain) expressions.
 
-Copyright 2014--2019 Michael Hayes, UCECE
+Copyright 2014--2020 Michael Hayes, UCECE
 
 """
 
 from __future__ import division
 from .fourier import inverse_fourier_transform
 from .expr import Expr
-from .sym import fsym, ssym, tsym, omegasym, j, pi
+from .sym import fsym, ssym, tsym, omegasym, omega0sym, j, pi
 
 
 class omegaExpr(Expr):
@@ -134,6 +134,7 @@ class Vomega(omegaExpr):
 
     quantity = 'Voltage spectrum'
     units = 'V/rad/s'
+    superkind = 'Voltage'    
 
     def __init__(self, val, **assumptions):
 
@@ -147,6 +148,7 @@ class Iomega(omegaExpr):
 
     quantity = 'Current spectrum'
     units = 'A/rad/s'
+    superkind = 'Current'        
 
     def __init__(self, val, **assumptions):
 
@@ -172,9 +174,11 @@ def omegaexpr(arg):
 
     if arg is omegasym:
         return omega
+    if arg is omega0sym:
+        return omega0    
     return omegaExpr(arg)
 
         
 from .texpr import Ht, It, Vt, Yt, Zt, tExpr
 omega = omegaExpr('omega')
-
+omega0 = omegaExpr('omega_0')
