@@ -27,6 +27,7 @@ Copyright 2014--2019 Michael Hayes, UCECE
 
 from __future__ import division
 from .functions import Heaviside, cos, exp
+from .sym import omega0sym
 from .symbols import j, t, s
 from .network import Network
 from .immitance import ImmitanceMixin
@@ -754,7 +755,7 @@ class G(OnePort):
             n1 = net.node
         if n2 == None:
             n2 = net.node
-        return 'R %s %s {%s}; right' % (n1, n2, 1 / self._G)
+        return 'R? %s %s {%s}; right' % (n1, n2, 1 / self._G)
 
 
 class L(OnePort):
@@ -932,9 +933,8 @@ class Vac(VoltageSource):
             phi = 0
             
         if omega is None:
-            from .symbols import omega
-        else:
-            omega = Expr(omega)
+            omega = omega0sym
+        omega = Expr(omega)
 
         V = Expr(V)
         phi = Expr(phi)
@@ -1060,9 +1060,8 @@ class Iac(CurrentSource):
             phi = 0
             
         if omega is None:
-            from .symbols import omega            
-        else:
-            omega = Expr(omega)
+            omega = omega0sym
+        omega = Expr(omega)
 
         I = Expr(I)
         phi = Expr(phi)
@@ -1275,13 +1274,34 @@ class P(O):
     """Port (open circuit)"""
     pass
 
+
+class Mass(L):
+    """Mass
+
+    Mass mval, initial velocity v0"""
+    pass
+
+
+class Spring(C):
+    """Spring
+
+    Spring constant kval, initial force f0"""
+    pass
+
+
+class Damper(R):
+    """Damper
+
+    Friction coeff rval"""
+    pass
+
     
 # Imports at end to circumvent circular dependencies
 from .expr import Expr
 from .cexpr import cExpr, Iconst, Vconst
 from .sexpr import sExpr, Is, Vs, Ys, Zs
 from .texpr import tExpr
-from .noiseexpr import Inoisy, Vnoisy
+from .noisy import Inoisy, Vnoisy
 from .voltage import Voltage
 from .current import Current
 from .phasor import Iphasor, Vphasor
