@@ -1,24 +1,41 @@
+"""This module provides the Phasor class to represent phasors for AC analysis.
+
+A phasor represents the amplitude and phase for a single sinusoid.  By
+default the angular frequency is omega_0 but it can be any number or
+symbol.
+
+Copyright 2014--2020 Michael Hayes, UCECE
+
+"""
+
 from __future__ import division
-from .sym import j
+from .sym import j, omega0sym
 from .functions import sin, cos, exp
 
-__all__ = ('Phasor', 'Vphasor', 'Iphasor')
+__all__ = ('Vphasor', 'Iphasor')
 
-from .omegaexpr import omegaExpr
+from .expr import Expr
 from .cexpr import cExpr
 
 
-class Phasor(omegaExpr):
-
-    # Could convert Vphasor + Vconst -> VSuper but that is not really
-    # the scope for types such as Vphasor and Vconst.
+class Phasor(Expr):
 
     def __init__(self, val, **assumptions):
 
         assumptions['positive'] = True  # ????
-        assumptions['ac'] = True        
+        assumptions['ac'] = True
+
+        if 'omega' not in assumptions:
+            assumptions['omega'] = omega0sym        
+        
         super (Phasor, self).__init__(val, **assumptions)
 
+    @property
+    def omega(self):
+        """Return angular frequency."""
+
+        return self.assumptions['omega']
+
     def __compat_add__(self, x, op):
 
         cls = self.__class__
@@ -98,15 +115,20 @@ class Phasor(omegaExpr):
     def rms(self):
         return {Vphasor: Vt, Iphasor : It}[self.__class__](0.5 * self)
 
-    # def plot(self, fvector=None, **kwargs):
+    def plot(self, **kwargs):
 
-    #     if self.omega != omegasym:
-    #         self.fourier.plot(fvector, **kwargs)
-    #     return omegaExpr(self).plot(fvector, **kwargs)
+        from .plot import plot_phasor
+        return plot_phasor(self, **kwargs)
 
 
 class Vphasor(Phasor):
-
+    """t-domain voltage (units V) parameterized as a phasor
+    of a single angular frequency, omega0."""
+        
+    quantity = 'Voltage'
+    units = 'V'
+    superkind = 'Voltage'    
+    
     def __init__(self, val, **assumptions):
 
         super(Vphasor, self).__init__(val, **assumptions)
@@ -118,6 +140,12 @@ class Vphasor(Phasor):
 
     
 class Iphasor(Phasor):
+    """t-domain current (units V) parameterized as a phasor
+    of a single angular frequency, omega0."""    
+
+    quantity = 'Current'
+    units = 'A'
+    superkind = 'Current'
 
     def __init__(self, val, **assumptions):
         super(Iphasor, self).__init__(val, **assumptions)
