@@ -9,16 +9,16 @@ import numpy as np
 # Perhaps add Formatter classes that will produce the plot data?
 
 
-def make_axes(figsize=None, axes=None):
+def make_axes(figsize=None, axes=None, **kwargs):
 
     from matplotlib.pyplot import subplots
     
     if axes is not None:
         fig = axes.figure
     elif figsize is not None :
-        fig, axes = subplots(1, figsize=figsize)
+        fig, axes = subplots(1, figsize=figsize, **kwargs)
     else:
-        fig, axes = subplots(1)
+        fig, axes = subplots(1, **kwargs)
 
     return axes
 
@@ -129,9 +129,10 @@ def plot_frequency(obj, f, **kwargs):
         else:
             f = np.linspace(f[0], f[1], npoints)            
 
+    plot_type = kwargs.pop('plot_type', 'dB-phase')
+            
     if not hasattr(obj, 'part'):
         obj2 = None        
-        plot_type = kwargs.pop('plot_type', 'dB-phase')
         if plot_type in ('dB_phase', 'dB-phase'):
             obj1 = obj.magnitude.dB
             if obj.is_complex:
@@ -272,3 +273,16 @@ def plot_sequence(obj, n, **kwargs):
         ax.set_ylabel(ylabel)
     ax.grid(True)
     return ax
+
+
+def plot_phasor(obj, **kwargs):
+
+    ax = make_axes(figsize=kwargs.pop('figsize', None),
+                   axes=kwargs.pop('axes', None),
+                   subplot_kw=dict(polar=True))
+
+    phi = obj.phase.val
+    mag = obj.magnitude.val
+    
+    ax.plot((phi, phi), (0, mag), **kwargs)
+    return ax
