@@ -1,7 +1,7 @@
 """
 This module provides support for rational functions.
 
-Copyright 2016--2019 Michael Hayes, UCECE
+Copyright 2016--2020 Michael Hayes, UCECE
 """
 
 from __future__ import division
@@ -128,6 +128,9 @@ def as_numer_denom_poly(expr, var):
 
 def as_numer_denom(expr, var):
 
+    if expr.has(1 / var):
+        expr = expr.cancel()
+    
     N = sym.S.One
     D = sym.S.One    
     for f in expr.as_ordered_factors():
@@ -376,18 +379,40 @@ class Ratfun(object):
         
         D = [(var - p.expr) ** o for p, o in zip(poles, occurrences)]
         denom = sym.Mul(K, *D)
+
+        # Could calculate all residues simultaneously using
+        # system of linear equations.
         
-        d = sym.limit(denom, var, pole)
+        def method1(numer, denom, var, pole):
         
-        if d != 0:
+            d = sym.limit(denom, var, pole)
+            
+            if d != 0:
+                tmp = (numer / denom).simplify()
+                
+                # Sometimes this takes ages...
+                return sym.limit(tmp, var, pole)
+
+            # Use l'Hopital's rule
             tmp = numer / denom
+            tmp = sym.diff(tmp, var)
+
             return sym.limit(tmp, var, pole)
 
-        # Use l'Hopital's rule
-        tmp = numer / denom
-        tmp = sym.diff(tmp, var)
-        
-        return sym.limit(tmp, var, pole)
+        def method2(numer, denom, var, pole):
+
+            d = denom.subs(var, pole)            
+            n = numer.subs(var, pole)
+            
+            if d != 0:
+                return n / d
+
+            ddenom = sym.diff(demon, var)
+            return n / ddenom.subs(pole)
+
+        m2 = method2(numer, denom, var, pole)        
+        return m2
+
 
     @property
     def numerator_denominator(self):
@@ -495,7 +520,7 @@ class Ratfun(object):
 
         return expr * undef
 
-    def partfrac(self, combine_conjugates=False, damping=None):
+    def partfrac(self, combine_conjugates=False, damping=None, split=True):
         """Convert rational function into partial fraction form.
 
         If combine_conjugates is True then the pair of partial
@@ -507,13 +532,14 @@ class Ratfun(object):
         try:
             Q, R, D, delay, undef = self.as_QRD(combine_conjugates, damping)
         except ValueError:
+            if not split:
+                raise
+            
             # Try splitting into terms
             result = 0
             for term in self.expr.as_ordered_terms():
-                try:
-                    result += Ratfun(term, self.var).partfrac(combine_conjugates)
-                except ValueError:
-                    result += term
+                result += Ratfun(term, self.var).partfrac(combine_conjugates,
+                                                          split=False)
             return result           
 
         result = Q
@@ -526,7 +552,7 @@ class Ratfun(object):
         result *= undef
         return result
 
-    def standard(self):
+    def standard(self, split=True):
         """Convert rational function into mixed fraction form.
 
         This is the sum of strictly proper rational function and a
@@ -536,15 +562,14 @@ class Ratfun(object):
 
         try:
             Q, M, D, delay, undef = self.as_QMD()
-
         except ValueError:
+            if not split:
+                raise
+            
             # Try splitting into terms
             result = 0
             for term in self.expr.as_ordered_terms():
-                try:
-                    result += Ratfun(term, self.var).standard()
-                except ValueError:
-                    result += term
+                result += Ratfun(term, self.var).standard(False)
             return result                       
         
         expr = Q + sym.cancel(M / D, self.var)
@@ -641,7 +666,7 @@ class Ratfun(object):
         return Dpoly.degree()    
     
     @property
-    def strictly_proper(self):
+    def is_strictly_proper(self):
         """Return True if the degree of the dominator is greater
         than the degree of the numerator."""
 
