@@ -1,3 +1,10 @@
+"""This module provides classes for schematic layout.
+
+Copyright 2014--2020 Michael Hayes, UCECE
+
+"""
+
+
 from __future__ import print_function
 
 def unique(alist):
@@ -33,6 +40,7 @@ class Cnodes(dict):
 
 
 class Gedge(object):
+    """Edge between common nodes"""
 
     def __init__(self, cpt, from_gnode, to_gnode, size, stretch=False):
 
@@ -57,6 +65,7 @@ class Gedge(object):
 
 
 class Gnode(object):
+    """Drawing node"""
 
     def __init__(self, name):
 
@@ -101,6 +110,7 @@ class Gnode(object):
 
     
 class Graph(dict):
+    """Drawing graph"""
 
     def __init__(self, name, nodes, debug=False):
 
@@ -398,7 +408,11 @@ class Graph(dict):
         try:
             return traverse(start)
         except RuntimeError:
-            msg = "The %s schematic graph is dodgy, probably a component is attached to the wrong nodes.\n" % self.name
+            # W a b; right
+            # W c d; right
+            # W a c; down
+            # W b d; up
+            msg = "The %s schematic graph has a loop.  For example, a node needs to be both above and below another node.  Probably a component is attached to the wrong nodes.\n" % self.name
             if self.debug:
                 msg += str(self)
             raise RuntimeError(msg)
