@@ -1,7 +1,7 @@
-"""This file provides the sExpr class to represent s-domain (Laplace
+"""This module provides the sExpr class to represent s-domain (Laplace
 domain) expressions.
 
-Copyright 2014--2019 Michael Hayes, UCECE
+Copyright 2014--2020 Michael Hayes, UCECE
 
 """
 
@@ -87,6 +87,11 @@ class sExpr(Expr):
             result = tExpr(result)
         return result
 
+    def ILT(self, **assumptions):
+        """Convert to t-domain.   This is an alias for inverse_laplace."""
+
+        return self.inverse_laplace(**assumptions)
+    
     def time(self, **assumptions):
         """Convert to time domain."""
         
@@ -99,7 +104,7 @@ class sExpr(Expr):
             assumptions = self.assumptions.copy()
         
         return self.__class__(self, **assumptions)
-
+    
     def fourier(self, **assumptions):
         """Convert to Fourier domain."""
         from .symbols import f
@@ -211,6 +216,33 @@ class sExpr(Expr):
 
         return N, D, delay
 
+    def differential_equation(self, input='x', output='y'):
+        """Create differential equation from transfer function. 
+
+        For example,  
+        >>> H = (s + 3) / (s**2 + 4)  
+        >>> H.differential_equation()
+                 d                    d       
+        3.y(t) + --(y(t)) = 4.x(t) + ---(x(t))
+                 dt                    2      
+                                     dt       
+        """
+
+        H = self
+        x = texpr('%s(t)' % input)
+        y = texpr('%s(t)' % output)
+
+        X = x.LT()
+        Y = y.LT()
+
+        N = self.N
+        D = self.D
+        
+        lhs = (N * Y).ILT(causal=True)
+        rhs = (D * X).ILT(causal=True)
+
+        return tExpr(sym.Eq(lhs.expr, rhs.expr))
+
     def evaluate(self, svector=None):
 
         return super(sExpr, self).evaluate(svector)
@@ -224,6 +256,12 @@ class sExpr(Expr):
     def parameterize(self, zeta=True):
         """Parameterize first and second-order expressions.
 
+        For example, pexpr, defs = expr.parameterize()
+
+        If parameterization is successful, defs is a dictionary
+        of the paramters.  The original expression can be obtained
+        with pexpr.subs(defs)
+
         For first order systems, parameterize as:
 
         K * (s + beta) / (s + alpha)
@@ -247,34 +285,48 @@ class sExpr(Expr):
         """
 
         def def1(defs, symbolname, value):
+            from .cexpr import cExpr
+            
             sym1 = symbol(symbolname)
-            defs[symbolname] = value
+            defs[symbolname] = cExpr(value)
             return sym1
 
-        ndegree = self.N.degree        
-        ddegree = self.D.degree
-        ncoeffs = self.N.coeffs(norm=True)
-        dcoeffs = self.D.coeffs(norm=True)
+        factors = self.as_ordered_factors()
 
+        spowers = [s**-4, s**-3, s**-2, s**-1, s, s**2, s**3, s**4]
+        for spower in spowers:
+            if spower in factors:
+                result, defs = (self / spower).parameterize(zeta)
+                return result * spower, defs
+        
+        N = self.N
+        D = self.D
+        
+        ndegree = N.degree        
+        ddegree = D.degree
+        ncoeffs = N.coeffs(norm=True)
+        dcoeffs = D.coeffs(norm=True)
+
+        result = None
         defs = ExprDict()
 
         K = self.K
         if ndegree < 1 and ddegree < 1:
-            return self, defs
-        if ndegree == 1 and ddegree == 1:
+            result = self
+        elif ndegree == 1 and ddegree == 1:
             K = def1(defs, 'K', K)
             alpha = def1(defs, 'alpha', dcoeffs[1])
             beta = def1(defs, 'beta', ncoeffs[1])
-            return K * (s + beta) / (s + alpha), defs
-        if ndegree == 1 and ddegree == 0:
+            result = K * (s + beta) / (s + alpha)
+        elif ndegree == 1 and ddegree == 0:
             K = def1(defs, 'K', K)
             beta = def1(defs, 'beta', ncoeffs[1])
-            return K * (s + beta), defs
-        if ndegree == 0 and ddegree == 1:
+            result = K * (s + beta)
+        elif ndegree == 0 and ddegree == 1:
             K = def1(defs, 'K', K)
             alpha = def1(defs, 'alpha', dcoeffs[1])
-            return K / (s + alpha), defs
-        if ddegree == 2:
+            result = K / (s + alpha)
+        elif ddegree == 2:
             K = def1(defs, 'K', K)
             coeffs = self.N.coeffs()
 
@@ -282,22 +334,27 @@ class sExpr(Expr):
                 sigma1 = def1(defs, 'sigma_1', dcoeffs[1] / 2)
                 omega1 = def1(defs, 'omega_1',
                               sqrt(dcoeffs[2] - (dcoeffs[1] / 2)**2).simplify())
-                return K * (self.N / coeffs[0]) / (s**2 + 2 * sigma1 * s + sigma1**2 + omega1**2), defs
-                
-            omega0 = def1(defs, 'omega_0', sqrt(dcoeffs[2]))
-            zeta = def1(defs, 'zeta', dcoeffs[1] / (2 * sqrt(dcoeffs[2])))
-            return K * (self.N / coeffs[0]) / (s**2 + 2 * zeta * omega0 * s + omega0**2), defs
-        
-        return self, defs
+                result = K * (self.N / coeffs[0]) / (s**2 + 2 * sigma1 * s + sigma1**2 + omega1**2)
+            else:
+                omega0 = def1(defs, 'omega_0', sqrt(dcoeffs[2]))
+                zeta = def1(defs, 'zeta', dcoeffs[1] / (2 * sqrt(dcoeffs[2])))
+                result = K * (self.N / coeffs[0]) / (s**2 + 2 * zeta * omega0 * s + omega0**2)
+
+        if result is None:
+            # Copy?
+            result = self
+
+        return self.__class__(result, **self.assumptions), defs
 
     def bilinear_transform(self):
 
+        # TODO: add frequency warping as an option
+
         from .discretetime import z, dt
 
         # s = ln(z) / dt gives the exact solution
         
         return self.subs((2 / dt) * (1 - z**-1) / (1 + z**-1))
-        
     
     
 # Perhaps use a factory to create the following classes?
@@ -370,6 +427,7 @@ class Vs(sExpr):
 
     quantity = 's-Voltage'
     units = 'V/Hz'
+    superkind = 'Voltage'
 
     def __init__(self, val, **assumptions):
 
@@ -387,6 +445,7 @@ class Is(sExpr):
 
     quantity = 's-Current'
     units = 'A/Hz'
+    superkind = 'Current'
 
     def __init__(self, val, **assumptions):
 
@@ -462,6 +521,6 @@ def sexpr(arg):
     return sExpr(arg)
 
 
-from .texpr import Ht, It, Vt, Yt, Zt, tExpr
+from .texpr import Ht, It, Vt, Yt, Zt, tExpr, texpr
 s = sExpr('s')
 
