@@ -1,6 +1,6 @@
 from __future__ import division
 from .expr import Expr, ExprDict, expr
-from .sym import tsym, omegasym, symbols_find, is_sympy, symsymbol
+from .sym import tsym, omega0sym, symbols_find, is_sympy, symsymbol
 from .acdc import is_ac
 from .printing import pprint, pretty, latex
 import six
@@ -8,7 +8,7 @@ import six
 __all__ = ('Super', 'Voltage', 'Current')
 
 class Super(ExprDict):
-    """This class represents a superposition of different signal types:
+    """This class represents a superposition of different signal types,
     DC, AC, transient, and noise.
     
     The time-domain representation is returned with the time method,
@@ -19,7 +19,7 @@ class Super(ExprDict):
     V.laplace() or with the notation V(s).  This does not include the
     noise component.
 
-    Noise components with different noise indentifiers are stored
+    Noise components with different noise identifiers are stored
     separately, keyed by the noise identifier.  They are ignored by
     the laplace() and time() methods.
 
@@ -109,7 +109,7 @@ class Super(ExprDict):
 
         keys = []
         for key in self.decompose().keys():
-            if not isinstance(key, str) or key is 'w':
+            if not isinstance(key, str) or key == 'w':
                 keys.append(key)
         return keys
 
@@ -118,7 +118,7 @@ class Super(ExprDict):
 
         keys = []
         for key in self.keys():
-            if isinstance(key, str) and key[0] is 'n':
+            if isinstance(key, str) and key[0] == 'n':
                 keys.append(key)
         return keys    
 
@@ -359,7 +359,7 @@ class Super(ExprDict):
         'laplace' :  the laplace domain representation (equivalent to self.laplace())
         'ivp' :  the s-domain representation (equivalent to self.laplace())
         'dc' : the DC component
-        'ac' : the AC component with angular frequency omega
+        'ac' : the AC component with angular frequency omega0
         omega : the AC component with angular frequency omega
         's' : the transient component in the s-domain
         'n' : the noise component
@@ -367,14 +367,14 @@ class Super(ExprDict):
               include the DC and AC components).
 
         """
-        if kind is 'super':
+        if kind == 'super':
             return self
-        elif kind is 'time':
+        elif kind == 'time':
             return self.time()
         elif kind in ('ivp', 'laplace'):
             return self.laplace()
 
-        if isinstance(kind, str) and kind[0] is 'n':
+        if isinstance(kind, str) and kind[0] == 'n':
             if kind not in self:
                 return self.decompose_domains['n'](0)
             return self[kind]
@@ -394,7 +394,7 @@ class Super(ExprDict):
     def netval(self, kind):
 
         def kind_keyword(kind):
-            if isinstance(kind, str) and kind[0] is 'n':
+            if isinstance(kind, str) and kind[0] == 'n':
                 return 'noise'
             elif kind in ('ivp', 'laplace'):
                 return 's'
@@ -408,17 +408,17 @@ class Super(ExprDict):
         if kind in ('s', 'ivp') and (val.is_causal or val.is_dc or val.is_ac):
             # Convert to time representation so that can re-infer
             # causality, etc.
-            return '{%s}' % val.time()
+            return val.time()
 
         keyword = kind_keyword(kind)
         
         if 'nid' in val.assumptions:
-            return '%s {%s} %s' % (keyword, val, val.nid)
+            return keyword, val, val.nid
 
-        if keyword is 'ac':
-            return '%s {%s} {%s} {%s}' % (keyword, val, 0, val.omega)
+        if keyword == 'ac':
+            return keyword, val, 0, val.omega
 
-        return '%s {%s}' % (keyword, val)
+        return keyword, val
     
     def _kind(self, value):
         if isinstance(value, Phasor):
@@ -560,8 +560,9 @@ class Super(ExprDict):
 
     @property
     def w(self):
-        """Return the AC component with angular frequency omega."""        
-        return self.select(omegasym)    
+        """Return the AC component with angular frequency omega0.
+        This should be deprecated."""
+        return self.select(omega0sym)    
 
     def time(self, **assumptions):
         """Convert to time domain."""
