@@ -1,4 +1,4 @@
-"""This file contains functions for handling symbols and in particular
+"""This module contains functions for handling symbols and in particular
 for integrating with SymPy.
 
 Copyright 2014--2020 Michael Hayes, UCECE
@@ -19,7 +19,7 @@ import sympy as sym
 import re
 from .state import state
 
-__all__ = ('symsymbol', 'sympify', 'simplify')
+__all__ = ('symsymbol', 'sympify', 'simplify', 'symbol_delete')
 
 
 global_dict = {}
@@ -280,6 +280,13 @@ def symdebug(expr, s='', indent=0):
     return s
 
 
+def symbol_delete(sym):
+    """Delete symbol.  This is useful if a symbol needs to be redefined
+    with different assumptions."""
+    
+    state.context.symbols.pop(sym)
+    
+
 def symbol_map(name):
 
     new = name
@@ -303,6 +310,7 @@ ssym = symsymbol('s', real=False)
 tsym = symsymbol('t', real=True)
 fsym = symsymbol('f', real=True)
 omegasym = symsymbol('omega', real=True)
+omega0sym = symsymbol('omega_0', real=True)
 
 pi = sym.pi
 j = sym.I
