@@ -4,10 +4,7 @@ import unittest
 
 
 class LcapyTester(unittest.TestCase):
-
-    """Unit tests for lcapy
-
-    """
+    """Unit tests for lcapy"""
 
     def assertEqual2(self, ans1, ans2, comment):
 
@@ -68,7 +65,8 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual2(a.Ndegree, 1, "N degree incorrect.")
         self.assertEqual2(a.Ddegree, 1, "D degree incorrect.")
         self.assertEqual2(a.degree, 1, "Degree incorrect.")
-        self.assertEqual2(a.strictly_proper, False, "strictly proper incorrect.")        
+        self.assertEqual2(a.is_rational_function, True, "is_rational_function incorrect.")                
+        self.assertEqual2(a.is_strictly_proper, False, "is_strictly_proper incorrect.")        
 
         self.assertEqual2(sorted(a.poles()), [2], "poles incorrect.")
         self.assertEqual2(sorted(a.zeros()), [-2], "zeros incorrect.")
@@ -117,7 +115,7 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual2(a.Ndegree, 2, "N degree incorrect.")
         self.assertEqual2(a.Ddegree, 1, "D degree incorrect.")
         self.assertEqual2(a.degree, 2, "Degree incorrect.")
-        self.assertEqual2(a.strictly_proper, False, "strictly proper incorrect.")
+        self.assertEqual2(a.is_strictly_proper, False, "is_strictly_proper incorrect.")
 
         self.assertEqual2(sorted(a.poles()), [2], "poles incorrect.")
         self.assertEqual2(sorted(a.zeros()), sorted([-3, -2]), "zeros incorrect.")
@@ -143,7 +141,7 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual2(a.N, 1, "N incorrect.")
         self.assertEqual2(a.D, (s - j) * (s + j), "D incorrect.")
 
-        self.assertEqual2(set(a.poles()), set([-j, j]), "poles incorrect.")
+        self.assertEqual2(set(a.poles()), set(expr([-j, j])), "poles incorrect.")
         self.assertEqual2(a.zeros(), {}, "zeros incorrect.")
 
         # This depends on if 2 * (s + j) is expanded to 2 * s + 2 * j
@@ -168,7 +166,7 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual2(a.N, 1, "N incorrect.")
         self.assertEqual2(a.D, (s - j) * (s + j), "D incorrect.")
 
-        self.assertEqual2(set(a.poles()), set([-j, j]), "poles incorrect.")
+        self.assertEqual2(set(a.poles()), set(expr([-j, j])), "poles incorrect.")
         self.assertEqual2(len(a.zeros()), 0, "zeros incorrect.")
 
         # This depends on if 2 * (s + j) is expanded to 2 * s + 2 * j
@@ -406,12 +404,6 @@ class LcapyTester(unittest.TestCase):
         self.assertEqual(Heaviside(2 * t - 1).is_causal, True, "Heaviside(2 * t - 1).is_causal")
         self.assertEqual(Heaviside(-2 * t - 1).is_causal, False, "Heaviside(-2 * t - 1).is_causal")
         
-    def test_noise(self):
-        a = Vnoisy(2)
-        self.assertEqual(a.nid, a.conjugate.nid, "Different nids for conjugate")
-        self.assertEqual(a.nid, a.real.nid, "Different nids for real")
-        self.assertEqual(a.nid, a.imag.nid, "Different nids for imag")
-        
     def test_has(self):
 
         a = Expr('3 * exp(-t) * t * a')
@@ -599,3 +591,32 @@ class LcapyTester(unittest.TestCase):
 
         self.assertEqual(F.mixedfrac(), F,  "undef delay sum mixedfrac")
         
+    def force_causal(self):
+
+        X = 1 / (1 + s)
+        x = X(t)
+
+        self.assertEqual(x.force_causal(), exp(-t) * Heaviside(t), "force causal")
+
+        x = X(t, causal=True)
+        self.assertEqual(x.force_causal(), exp(-t) * Heaviside(t), "force causal if already causal")
+        
+
+    def strip_condition(self):
+
+        X = 1 / (1 + s)
+        x = X(t)
+
+        self.assertEqual(x.strip_condition(), exp(-t), "remove condition")
+
+        x = X(t, condition=True)
+        self.assertEqual(x.strip_condition(), exp(-t), "remove condition if causal")
+        
+        
+    def test_val(self):
+
+        a = expr('3 / 4')
+
+        self.assertEqual(a.val, 0.75, "val")
+        self.assertEqual(a.evalf(), 0.75, "evalf")
+        
