@@ -1,6 +1,6 @@
-"""This file provides the tExpr class to represent time domain expressions.
+"""This module provides the tExpr class to represent time domain expressions.
 
-Copyright 2014--2019 Michael Hayes, UCECE
+Copyright 2014--2020 Michael Hayes, UCECE
 
 """
 
@@ -11,6 +11,7 @@ from .sym import fsym, ssym, tsym, j, oo
 from .acdc import ACChecker, is_dc, is_ac, is_causal
 from .laplace import laplace_transform
 from .fourier import fourier_transform
+import sympy as sym
 
 __all__ = ('Ht', 'It', 'Vt', 'Yt', 'Zt')
 
@@ -53,12 +54,6 @@ class tExpr(Expr):
         if is_causal(self, var):
             self.assumptions['causal'] = True
 
-    def merge_assumptions(self, **assumptions):
-        
-        new_assumptions = self.assumptions.copy()
-        new_assumptions.update(assumptions)
-        return new_assumptions
-
     def laplace(self, **assumptions):
         """Determine one-sided Laplace transform with 0- as the lower limit."""
 
@@ -76,6 +71,11 @@ class tExpr(Expr):
             result = sExpr(result, **assumptions)
         return result
 
+    def LT(self, **assumptions):
+        """Convert to s-domain.   This is an alias for laplace."""
+
+        return self.laplace(**assumptions)
+    
     def fourier(self, **assumptions):
         """Attempt Fourier transform."""
 
@@ -89,6 +89,11 @@ class tExpr(Expr):
             result = fExpr(result **self.assumptions)
         return result
 
+    def FT(self, **assumptions):
+        """Convert to f-domain.   This is an alias for fourier."""
+
+        return self.fourier(**assumptions)    
+
     def phasor(self, **assumptions):
 
         check = ACChecker(self, t)
@@ -132,8 +137,31 @@ class tExpr(Expr):
         """Determine value at t = oo."""
 
         return self.__class__(sym.limit(self.expr, self.var, oo))
-    
 
+    def remove_condition(self):
+        """Remove the piecewise condition from the expression.
+        See also force_causal."""
+
+        if not self.is_conditional:
+            return self
+        expr = self.expr
+        expr = expr.args[0].args[0]
+        return self.__class__(expr)
+
+    def force_causal(self):
+        """Remove the piecewise condition from the expression
+        and multiply by Heaviside function.  See also remove_condition."""
+
+        if self.is_causal:
+            return self
+        
+        expr = self.expr
+        if self.is_conditional:
+            expr = expr.args[0].args[0]            
+        expr = expr * sym.Heaviside(t)
+        return self.__class__(expr)        
+
+    
 class Yt(tExpr):
 
     """t-domain 'admittance' value."""
@@ -166,6 +194,7 @@ class Vt(tExpr):
 
     quantity = 'Voltage'
     units = 'V'
+    superkind = 'Voltage'        
 
     def __init__(self, val, **assumptions):
 
@@ -176,13 +205,15 @@ class Vt(tExpr):
     def cpt(self):
         from .oneport import V
         return V(self)
-        
+
+    
 class It(tExpr):
 
     """t-domain current (units A)."""
 
     quantity = 'Current'
     units = 'A'
+    superkind = 'Current'    
 
     def __init__(self, val, **assumptions):
 
