@@ -110,10 +110,25 @@ def ztransform_sum(expr, n, z):
     return F1 * F2
 
 
+def remove_heaviside(expr, n):
+
+    rest = sym.S.One
+    for factor in expr.as_ordered_factors():
+        if (factor.is_Function and factor.func == sym.Heaviside and
+            factor.args[0] == n):
+            # Could remove Heaviside[n+m] where m > 0
+            pass
+        else:
+            rest *= factor
+    return rest
+
+
 def ztransform_term(expr, n, z):
 
     const, expr = factor_const(expr, n)
 
+    expr = remove_heaviside(expr, n)
+    
     if expr.has(sym.Sum):
         try:
             return ztransform_sum(expr, n, z) * const
@@ -162,7 +177,7 @@ def ztransform_term(expr, n, z):
         if args[0] is n:
             result = 1 / (1 - invz)
         else:
-            delay = args[0] - n
+            delay = n - args[0]
             if not delay.has(n):
                 result = invz ** delay * 1 / (1 - invz)
 
@@ -176,6 +191,16 @@ def ztransform_term(expr, n, z):
         if aexpr == n:
             result = (sym.sin(aconst) * invz) / (1 - 2 * sym.cos(aconst) * invz + invz ** 2)            
 
+    # Handle a**n
+    elif expr.is_Pow and expr.args[1] == n:
+        # TODO, handle n + m, etc.
+        result = 1 / (1 - expr.args[0] * invz)
+
+    # Handle a**(-n)
+    elif (expr.is_Pow and expr.args[1].is_Mul and
+          expr.args[1].args[0] == -1 and expr.args[1].args[1] == n):
+        result = 1 / (1 - (1 / expr.args[0]) * invz)          
+            
     if result is None:
         # Use m instead of n to avoid n and z in same expr.
         # TODO, check if m already used...
@@ -184,7 +209,7 @@ def ztransform_term(expr, n, z):
         zsym = sympify(str(z))
         result = sym.Sum(expr.subs(nsym, msym) * zsym**msym, (msym, 0, sym.oo))
         
-    return result * const
+    return const * result
 
 
 def ztransform(expr, n, z):
@@ -198,9 +223,10 @@ def ztransform(expr, n, z):
         return sym.Eq(ztransform(expr.args[0], n, z),
                       ztransform(expr.args[1], n, z))
 
+    const, expr = factor_const(expr, n)    
     key = (expr, n, z)
     if key in ztransform_cache:
-        return ztransform_cache[key]
+        return const * ztransform_cache[key]
 
     if expr.has(z):
         raise ValueError('Cannot Z transform expression %s that depends on %s' % (expr, z))
@@ -234,44 +260,75 @@ def ztransform(expr, n, z):
 
     result = result.simplify()
     ztransform_cache[key] = result
-    return result
+    return (const * result).simplify()
 
 
 def inverse_ztransform_ratfun(expr, z, n, **assumptions):
 
-    # Maybe better doing partial fraction expansion of X(z) / z
-    # the multiplying by z.
+    expr = expr / z
     
-    damping = assumptions.get('damping', None)
+    # Handle special case 1 / (z**m * (z - 1)) since this becomes u[n - m]
+    # The default method produces u[n] - delta[n] for u[n-1].  This is correct
+    # but can be simplified.
+    # In general, 1 / (z**m * (z - a)) becomes a**n * u[n - m]
 
-    invz = symsymbol('invz', real=False)
-    expr = expr.subs(z, 1 / invz)
+    if (len(expr.args) == 2 and expr.args[1].is_Pow and
+        expr.args[1].args[0].is_Add and
+        expr.args[1].args[0].args[0] == -1 and
+        expr.args[1].args[0].args[1] == z):
+
+        delay = None
+        if expr.args[0] == z:
+            delay = 1
+        elif expr.args[0].is_Pow and expr.args[0].args[0] == z:
+            a = expr.args[0].args[1]
+            if a.is_positive:
+                print('Warning, dodgy z-transform.  Have advance of unit step.')
+            elif not a.is_negative:
+                print('Warning, dodgy z-transform.  May have advance of unit step.')                
+            delay = -a
+        elif (expr.args[0].is_Pow and expr.args[0].args[0].is_Pow and
+              expr.args[0].args[0].args[0] == z and
+              expr.args[0].args[0].args[1] == -1):              
+            a = expr.args[0].args[1]
+            if a.is_negative:
+                print('Warning, dodgy z-transform.  Have advance of unit step.')
+            elif not a.is_positive:
+                print('Warning, dodgy z-transform.  May have advance of unit step.')                
+            delay = a            
+
+        if delay is not None:
+            return sym.Heaviside(n - delay), sym.S.Zero
+        
+    damping = assumptions.get('damping', None)
     
-    zexpr = Ratfun(expr, invz)
+    zexpr = Ratfun(expr, z)
 
     Q, M, D, delay, undef = zexpr.as_QMD()
 
-    result1 = sym.S.Zero
+    cresult = sym.S.Zero
+    uresult = sym.S.Zero
 
     if Q:
-        Qpoly = sym.Poly(Q, invz)        
+        Qpoly = sym.Poly(Q, z)        
         C = Qpoly.all_coeffs()
         for m, c in enumerate(C):
-            result1 += c * unitimpulse(n - len(C) + m + 1)
+            cresult += c * unitimpulse(n - len(C) + m + 1)
 
-    expr = M / D
+    # There is problem with determining residues if
+    # have 1/(z*(-a/z + 1)) instead of 1/(-a + z).  Hopefully,
+    # simplify will fix things...
+    expr = (M / D).simplify()
     for factor in expr.as_ordered_factors():
         if factor == sym.oo:
-            return factor
+            return factor, factor
 
-    zexpr = Ratfun(expr, invz)
+    zexpr = Ratfun(expr, z)
     poles = zexpr.poles(damping=damping)
     polesdict = {}
     for pole in poles:
         polesdict[pole.expr] = pole.n
     
-    result2 = sym.S.Zero
-
     for pole in poles:
 
         p = pole.expr
@@ -286,23 +343,32 @@ def inverse_ztransform_ratfun(expr, z, n, **assumptions):
             r = zexpr.residue(p, poles)
 
             # TODO combine conjugates to get a real result
-            
-            result2 -= (r * p ** (-n - 1)).simplify()
+            # See laplace.py
+
+            if p == 0:
+                cresult += r * unitimpulse(n)
+            else:
+                uresult += r * p ** n
             continue
 
         # Handle repeated poles.
-        expr2 = expr * (invz - p) ** o
+        
+        expr2 = expr * (z - p) ** o
         expr2 = expr2.simplify()
         for i in range(1, o + 1):
             m = o - i
             r = sym.limit(
-                sym.diff(expr2, invz, m), invz, p) / sym.factorial(m)
-            result2 += r * (p ** n) * n**(i - 1)
+                sym.diff(expr2, z, m), z, p) / sym.factorial(m)
 
-    # result1 is a sum of Dirac deltas and its derivatives so is known
+            if p == 0:
+                cresult += r * unitimpulse(n - i + 1)
+            else:            
+                uresult += r * (p ** n) * n**(i - 1)
+
+    # cresult is a sum of Dirac deltas and its derivatives so is known
     # to be causal.
 
-    return result1, result2
+    return cresult, uresult
 
 
 def dummyvar(intnum=0):
@@ -357,10 +423,10 @@ def inverse_ztransform_product(expr, z, n, **assumptions):
             result = sym.S.Zero
             for term in terms:
                 result += inverse_ztransform_product(factors[1] * term, z, n)
-            return result * const
+            return const * result
 
-    result1, result2 = inverse_ztransform_term1(factors[0], z, n)
-    result = result1 + result2
+    cresult, uresult = inverse_ztransform_term1(factors[0], z, n)
+    result = cresult + uresult
 
     intnum = 0
     for m in range(len(factors) - 1):
@@ -403,29 +469,41 @@ def inverse_ztransform_product(expr, z, n, **assumptions):
         # Convert product to convolution
         dummy = dummyvar(intnum)
         intnum += 1
-        result1, result2 = inverse_ztransform_term1(factors[m + 1], z, n)
-        expr2 = result1 + result2
+        cresult, uresult = inverse_ztransform_term1(factors[m + 1], z, n)
+        expr2 = cresult + uresult
         result = sym.Sum(result.subs(n, n - dummy) * expr2.subs(n, dummy),
                          (dummy, n1, n2))
     
-    return result * const
+    return const * result
 
 
 def inverse_ztransform_power(expr, z, n, **assumptions):
 
     # Handle expressions with a power of z.
-    if not (expr.is_Pow and expr.args[0] == z):
-        raise ValueError('Expression %s is not a power of z' % expr)
-    exponent = expr.args[1]
+    if (expr.is_Pow and expr.args[0] == z):
+        exponent = expr.args[1]
 
-    if exponent.is_positive:
-        # TODO, fix
-        return unitimpulse(n + exponent)
+        if exponent.is_positive:
+            print('Warning, dodgy z-transform.  Have advance of unit impulse.')
+        elif not exponent.is_negative:
+            print('Warning, dodgy z-transform.  May have advance of unit impulse.')
+        
+        return unitimpulse(n + exponent), sym.S.Zero
 
-    if exponent.is_negative:
-        return unitimpulse(n + exponent)
+    # Handle expressions with a power of (1 / z).
+    if (expr.is_Pow and expr.args[0].is_Pow and
+        expr.args[0].args[0] == z and expr.args[0].args[1] == -1):
 
-    raise ValueError('Cannot determine sign of exponent for %s' % expr)
+        exponent = expr.args[1]
+
+        if exponent.is_negative:
+            print('Warning, dodgy z-transform.  Have advance of unit impulse.')
+        elif not exponent.is_positive:
+            print('Warning, dodgy z-transform.  May have advance of unit impulse.')
+        
+        return unitimpulse(n - exponent), sym.S.Zero        
+
+    raise ValueError('Expression %s is not a power of z' % expr)
 
 
 def inverse_ztransform_sympy(expr, z, n):
@@ -452,16 +530,27 @@ def inverse_ztransform_term1(expr, z, n, **assumptions):
         # that the unknown functions are causal.  Note ztransform_func
         # just changes the name so it works as inverse_ztransform_func.
         result = ztransform_func(expr, z, n, True)
-        return result * const, sym.S.Zero
+        return const * result, sym.S.Zero
     
     if expr.has(sym.function.AppliedUndef):
         return const * inverse_ztransform_product(expr, z, n,
-                                               **assumptions), sym.S.Zero
+                                                  **assumptions), sym.S.Zero
 
+    if expr == z:
+        print('Warning, dodgy z-transform.  Have advance of unit impulse.') 
+        return const * unitimpulse(n + 1), sym.S.Zero        
+
+    if (expr.is_Pow and
+        (expr.args[0] == z or
+         (expr.args[0].is_Pow and
+          expr.args[0].args[0] == z and expr.args[0].args[1] == -1))):
+        cresult, uresult = inverse_ztransform_power(expr, z, n, **assumptions)
+        return const * cresult, const * uresult        
+        
     try:
         # This is the common case.
-        result1, result2 = inverse_ztransform_ratfun(expr, z, n, **assumptions)
-        return const * result1, const * result2
+        cresult, uresult = inverse_ztransform_ratfun(expr, z, n, **assumptions)
+        return const * cresult, const * uresult
     except:
         pass
 
@@ -470,35 +559,85 @@ def inverse_ztransform_term1(expr, z, n, **assumptions):
     except:
         pass
 
-    if expr.is_Pow and expr.args[0] == z:
-        return sym.S.Zero, const * inverse_ztransform_power(expr, z, n)
-    
     # As last resort see if can convert to convolutions...
     return sym.S.Zero, const * inverse_ztransform_product(expr, z, n)
     
     
 def inverse_ztransform_term(expr, z, n, **assumptions):
 
-    result1, result2 = inverse_ztransform_term1(expr, z, n, **assumptions)
+    cresult, uresult = inverse_ztransform_term1(expr, z, n, **assumptions)
 
     if assumptions.get('causal', False):
-        result2 = result2 * sym.Heaviside(n)
+        uresult = uresult * sym.Heaviside(n)
     
-    return result1, result2
+    return cresult, uresult
 
 
 def inverse_ztransform_by_terms(expr, z, n, **assumptions):
 
     terms = expr.as_ordered_terms()
 
-    result1 = sym.S.Zero
-    result2 = sym.S.Zero    
+    cresult = sym.S.Zero
+    uresult = sym.S.Zero    
 
     for term in terms:
         part1, part2 = inverse_ztransform_term(term, z, n, **assumptions)
-        result1 += part1
-        result2 += part2        
-    return result1, result2
+        cresult += part1
+        uresult += part2        
+    return cresult, uresult
+
+
+def inverse_ztransform_make(n, const, cresult, uresult, **assumptions):
+
+    result = const * (cresult + uresult)
+    
+    if assumptions.get('dc', False):
+        free_symbols = set([symbol.name for symbol in result.free_symbols])
+        if 'n' in free_symbols:
+            raise ValueError('Something wonky going on, expecting dc.')
+    
+    elif assumptions.get('ac', False):
+
+        if cresult != 0:
+            raise ValueError('Inverse z-transform weirdness for %s with is_ac True' % expr)
+        # TODO, perform more checking of the result.
+        
+    elif not assumptions.get('causal', False):
+
+        # Cannot determine result for n < 0
+        result = sym.Piecewise((result, n >= 0))
+            
+    return result    
+
+
+def inverse_ztransform1(expr, z, n, **assumptions):
+
+    const, expr = factor_const(expr, z)
+    
+    key = (expr, z, n, 
+           assumptions.get('causal', False),
+           assumptions.get('damping', None))
+    
+    if key in inverse_ztransform_cache:
+        cresult, uresult = inverse_ztransform_cache[key]        
+        return const, cresult, uresult        
+
+    try:
+        if expr.is_Add:
+            cresult, uresult = inverse_ztransform_by_terms(expr, z, n, **assumptions)
+        else:
+            try:
+                cresult, uresult = inverse_ztransform_term(expr, z, n, **assumptions)
+            except:
+                expr = sym.expand(expr)
+                cresult, uresult = inverse_ztransform_by_terms(expr, z, n,
+                                                               **assumptions)
+    except:
+        raise ValueError('Cannot determine z-transform of %s' % expr)
+        
+    # cresult is known to be causal, uresult is unsure
+    inverse_ztransform_cache[key] = cresult, uresult
+    return const, cresult, uresult    
 
 
 def inverse_ztransform(expr, z, n, **assumptions):
@@ -517,50 +656,11 @@ def inverse_ztransform(expr, z, n, **assumptions):
         return sym.Eq(inverse_ztransform(expr.args[0], z, n, **assumptions),
                       inverse_ztransform(expr.args[1], z, n, **assumptions))
 
-    # TODO, simplify
-    key = (expr, z, n, assumptions.get('dc', False),
-           assumptions.get('ac', False),
-           assumptions.get('causal', False),
-           assumptions.get('damping', None))
-    
-    if key in inverse_ztransform_cache:
-        return inverse_ztransform_cache[key]
-
     if expr.has(n):
         raise ValueError('Cannot inverse z-transform  %s that depends on %s' % (expr, t))
-    
-    if assumptions.get('dc', False):
-        result = expr * (z - 1) / z
-            
-        free_symbols = set([symbol.name for symbol in result.free_symbols])
-        if 'z' in free_symbols:
-            raise ValueError('Something wonky going on, expecting dc.')
-        return result
 
-    if expr.is_Add:
-        result1, result2 = inverse_ztransform_by_terms(expr, z, n, **assumptions)
-    else:
-        try:
-            result1, result2 = inverse_ztransform_term(expr, z, n, **assumptions)
-        except:
-            expr = sym.expand(expr)
-            result1, result2 = inverse_ztransform_by_terms(expr, z, n,
-                                                           **assumptions)
-        
-    # result1 is known to be causal, result2 is unsure
-
-    if assumptions.get('ac', False):
-        if result1 != 0:
-            raise ValueError('Inverse z-transform weirdness for %s'
-                             ' with is_ac True' % expr)
-        # TODO, perform more checking of the result.
-
-    elif not assumptions.get('causal', False) and  result2 != 0:
-        result2 = sym.Piecewise((result2, n >= 0))        
-
-    result = result1 + result2    
-    inverse_ztransform_cache[key] = result
-    return result
+    const, cresult, uresult = inverse_ztransform1(expr, z, n, **assumptions)
+    return inverse_ztransform_make(n, const, cresult, uresult, **assumptions)
 
 
 def ZT(expr, n, z, **assumptions):
